/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.dussan.vaadin.dtable.config.TableToolsPosition;
import org.dussan.vaadin.dtable.config.TooltipPosition;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;
import org.dussan.vaadin.dtable.properties.filter.Filter;
import org.dussan.vaadin.dtable.properties.order.Order;
import org.dussan.vaadin.dtable.properties.order.OrderType;
import org.dussan.vaadin.dtable.query.elements.Field;
import org.dussan.vaadin.dtable.query.elements.OrderBy;
import org.dussan.vaadin.dtable.query.elements.OrderByType;
import org.dussan.vaadin.dtable.translations.TranslatedWord;
import org.dussan.vaadin.dtable.windows.exporter.base.CompressionType;
import org.dussan.vaadin.dtable.windows.exporter.base.MsFileVersion;

public class DemoScrolableTable extends DTable {

	private static final long serialVersionUID = -6865664679839888794L;
	private int allRows = 11;
	private int allColumns = 10;

	public DemoScrolableTable() {
		super("demoScrollableTable");
		getTableConfig().setLocale(new Locale("sl", "SI"));
		getTableConfig().enableScrollableTable(true);
		getTableConfig().enableOrderedTable(true);
		getTableConfig().enableHighlightActiveRow(true);
		getTableConfig().enableOddAndEvenColoringRows(true);
		getTableConfig().setTableHeight(150);

		getTableConfig().enableAutoPrepareData(true);
		getTableConfig().enableAutoPrepareDataWithoutHiddenColumns(true);
		getTableConfig().enableAutoPrepareDataWithoutRemovedColumns(true);

		getTableConfig().enableFilteredTable(true);
		getTableConfig().enableFilterInfo(true);
		getTableConfig().enableFilteredTable(true);
		getTableConfig().enableContextMenuForFilteredTable(true);
		getTableConfig().setContextMenuDepthForFilteredTable(4);

		getTableConfig().enableQuickSearch(true);
		getTableConfig().enableQuickSearchFromatting(true);
		getTableConfig().enableQuickSearchImmediate(false);
		getTableConfig().enableQuickSearchHide(true);
		getTableConfig().enableQuickSearchVisibleAtStartup(true);
		getTableConfig().enableQuickSearchClearOnNoNewData(true);

		getTableConfig().setTableToolsPosition(TableToolsPosition.TOP_LEFT);
		getTableConfig().enableSearch(true);
		getTableConfig().enableSearchImmediate(false);
		getTableConfig().enableSearchHide(true);
		getTableConfig().enableSearchVisibleAtStartup(true);
		getTableConfig().enableSearchClearOnNoNewData(true);

		getTableConfig().enableFreezeUnfreezeColumns(false);
		getTableConfig().enableFreezeAnyColumns(true);
		getTableConfig().enableUnfreezeAnyColumns(true);
		getTableConfig().enableColumnUnfreeze(true, 2);

		getTableConfig().enableHideUnhideColumns(true);
		getTableConfig().enableHideAnyColumns(true);
		// getTableConfig().enableColumnsHide(true, 2, 4);
		getTableConfig().enableUnhideAnyColumns(true);
		// getTableConfig().enableColumnsUnhide(true, 2);
		getTableConfig().enableNewDataWhenHideUnhideColumn(true);

		getTableConfig().enableLoadingMessage(true);
		getTableConfig().setLoadingMessageOpacity(0.5f);
		getTableConfig().setLoadingMessageDelay(100, 300);

		getTableConfig().enableFrozenTable(false);
		getTableConfig().enableFrozenTableVisible(true);
		getTableConfig().enableOrderedFrozenTable(true);
		getTableConfig().enableFilteredFrozenTable(true);
		getTableConfig().enableHideFrozenTable(true);
		getTableConfig().enableResizeFrozenTable(true);
		getTableConfig().enableEmptyFrozenTable(true);
		getTableConfig().setTableFrozenMinimalWidth(200);

		getTableConfig().header().enableAllColumns(true);
		getTableConfig().header().enableVerticalLines(true);
		getTableConfig().header().enableExpandedFrozenCell(true, true);
		getTableConfig().header().enableExpandedFrozenCellForActiveColumn(true);
		getTableConfig().header().enableExpandedCell(true, true);
		getTableConfig().header().enableExpandedCellForActiveColumn(true);
		getTableConfig().header().setRowHeight(30);
		getTableConfig().header().enableWordWrap(false);
		getTableConfig().header().setDefaultColumnAlignment(ColumnAlign.CENTER);
		getTableConfig().header().enableTooltip(true);
		getTableConfig().header().enableWordWrapInTooltip(true);
		getTableConfig().header().setTooltipWidth(100, 230);
		getTableConfig().header().setTooltipPosition(TooltipPosition.BOTTOM);

		getTableConfig().body().enableVerticalLines(true);
		getTableConfig().body().enableExpandedFrozenCell(true, true);
		getTableConfig().body().enableExpandedCell(true, true);
		getTableConfig().body().enableWordWrap(false);
		getTableConfig().body().setRowHeight(24);
		// getTableConfig().body().setLineHeight(18);
		getTableConfig().body().setRowAlignment(RowAlign.MIDDLE);
		getTableConfig().body().setDefaultColumnAlignment(ColumnAlign.CENTER);

		getTableExporter().setCompression(CompressionType.ZIP);
		getTableExporter().enableExportCsvFile(true);
		getTableExporter().enableExportMdbFile(true);
		getTableExporter().enableTimestamp(true);
		getTableExporter().enableFormattedData(true);
		getTableExporter().setMsFileVersion(MsFileVersion.V2003);

		getTranslation().set(new Locale("en"), TranslatedWord.IS_EMPTY,
				"JE PRAZNO");
		getTranslation().set(new Locale("en"), TranslatedWord.IS_NOT_EMPTY,
				"NI PRAZNO");

		// getTableProperties().setFrozenColumns(0, 2, 4);
		getTableProperties().setFrozenColumns(2);
		getTableProperties().setHiddenColumns(7);
		getTableProperties().setRemovedColumns(1, 6, 9);
		getTableProperties().setOrders(OrderType.ASCEDING, 1, 6);
		getTableProperties().setOrders(OrderType.DESCEDING, 4, 5);

		for (int column = 1; column <= allColumns; column++) {
			if (column == 10) {
				setColumn(column - 1, "HEADER " + column, 100,
						new SimpleDateFormat("HH:mm:ss"));
			} else if (column == 11) {
				setColumn(column - 1, "HEADER " + column, "Datum",
						new SimpleDateFormat("dd.MM.yyyy HH:mm:ss"));
			} else if (column % 2 == 2) {
				setColumn(column - 1, "HEADER " + column, 50,
						ColumnAlign.RIGHT, ColumnAlign.RIGHT,
						new DecimalFormat("#0.00"));
			} else if (!(column == 1 || column == 3 || column == 7)) {
				setColumn(column - 1, "HEADER " + column, 200,
						ColumnAlign.LEFT, ColumnAlign.LEFT, new DecimalFormat(
								"#,##0.00"));
			} else {
				setColumn(column - 1, "HEADER " + column, "Denar", 50,
						ColumnAlign.RIGHT, ColumnAlign.RIGHT,
						new DecimalFormat("#,##0.00"));
			}
			getTableQuery().getQuery().addField(column - 1,
					new Field("HEADER" + column));
		}

		getTableQuery().getQuery().addFromTable("rpu.pu_2012_03_", "this0_");
		getTableQuery().getQuery().addGroupBy("HEADER2", "HEADER10");
		getTableQuery().getQuery().addOrderBy(
				new OrderBy("HEADER2", OrderByType.ASCEDING),
				new OrderBy("HEADER7", OrderByType.ASCEDING),
				new OrderBy("HEADER5", OrderByType.DESCEDING),
				new OrderBy("HEADER6", OrderByType.DESCEDING));
		/*
		 * String name = "test";
		 * getTableExporter().createColumnDefinition("test"); for (int column =
		 * 0; column < allColumns; column++) { if (column == 1 || column == 3 ||
		 * column == 7) { getTableExporter().setColumnDefinition(name, column,
		 * "HEADER " + (column + 1), null, ExportingColumnType.TEXT, 5); } else
		 * if (column == 9) { getTableExporter() .setColumnDefinition( name,
		 * column, "HEADER " + (column + 1), new Formatting(column, new
		 * SimpleDateFormat( "HH:mm:ss")), ExportingColumnType.DATE); } else if
		 * (column == 10) { getTableExporter().setColumnDefinition( name,
		 * column, "HEADER " + (column + 1), new Formatting(column, new
		 * SimpleDateFormat( "dd.MM.yyyy HH:mm:ss")), ExportingColumnType.DATE);
		 * } else if (column % 5 == 2) {
		 * getTableExporter().setColumnDefinition(name, column, "HEADER " +
		 * (column + 1), new Formatting(column, new DecimalFormat("#0.00")),
		 * ExportingColumnType.FLOAT); } else {
		 * getTableExporter().setColumnDefinition(name, column, "HEADER " +
		 * (column + 1), new Formatting(column, new DecimalFormat("#,##0.00")),
		 * ExportingColumnType.FLOAT, 4); } }
		 */
	}

	@Override
	public Object[] getRowsData(int startRow, int rows, Order[] sortings,
			Filter[] filterings, Filter[] quickSearch) {

		String sql = getTableQuery().getQuery().getQueryString();

		rows = rows == 0 ? allRows : rows;

		List<Object> data = new ArrayList<Object>();
		for (int row = startRow; row < (startRow + rows)
				&& row < getAllRows(filterings, quickSearch); row++) {
			for (int column = 0; column < allColumns; column++) {
				if (column == 1) {
					String url = getApplication().getURL()
							+ "VAADIN/widgetsets/org.dussan.vaadin.dtable.DtableWidgetset/dtable-css/images/sort_desc.png";
					data.add(setImage(url, 16, 16));
				} else if (column == 3 || column == 7) {
					data.add("Testing, testing, testing, testing, testing, ...");
				} else if (column == 9 || column == 10) {
					data.add(new Date());
				} else {
					if (row % 5 == 2) {
						data.add(null);
					} else {
						data.add((float) row + (float) column / 100);
					}
				}
			}
		}
		return data.toArray();
	}

	@Override
	public long getAllRows(Filter[] filterings, Filter[] quickSearch) {
		String sql = getTableQuery().getQuery().getAllRowsCounterQueryString();

		if (filterings != null && filterings.length > 2) {
			return 0;
		}
		return (filterings == null || filterings.length == 0 ? allRows
				: allRows / (filterings.length + 1));
	}

	@Override
	public boolean prepareExportedData(String name, Order[] sortings,
			Filter[] filterings, Filter[] quickSearch) {
		return true;
	}

	@Override
	public long getExportedAllRows(String name, Filter[] filterings,
			Filter[] quickSearch) {
		return allRows;
	}

	@Override
	public Object[] getExportedData(String name, int startRow, int rows,
			Order[] sortings, Filter[] filterings, Filter[] quickSearch) {
		List<Object> data = new ArrayList<Object>();
		for (int row = startRow; row < (startRow + rows); row++) {
			for (int column = 0; column < allColumns; column++) {
				if (column == 1) {
					String url = getApplication().getURL()
							+ "VAADIN/widgetsets/org.dussan.vaadin.dtable.DtableWidgetset/dtable-css/images/sort_desc.png";
					data.add(setImage(url, 16, 16));
				} else if (column == 9 || column == 10) {
					data.add(new Date());
				} else if (column == 3 || column == 7) {
					data.add("Testing, testing, testing, testing, testing, ...");
				} else {
					if (row % 5 == 2) {
						data.add(null);
					} else {
						data.add((float) row + (float) column / 100);
					}
				}
			}
		}
		return data.toArray();
	}

	@Override
	public void exportProcessCompleted() {
	}

}
