/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client;

import java.util.ArrayList;
import java.util.List;

import com.vaadin.terminal.gwt.client.ValueMap;

public class TableConfig {

	private int tableFrozenMinimalWidth = 0;
	private int tableMinimalWidth = 0;
	private int contextMenuDepthForFilteredFrozenTable = 0;
	private int contextMenuDepthForFilteredTable = 0;
	private int tableHeight = 0;
	private int pageSize = 0;
	private int pagerPages = 0;
	private String tableToolsPosition = "-top-right";
	private String filterInfoPosition = "bottom";

	private int headerRowsAlignment = 2;
	private int headerRowHeight = 26;
	private int headerLineHeight = 22;
	private int headerCellMarginTop = 2;
	private int headerCellMarginRight = 4;
	private int headerCellMarginBottom = 2;
	private int headerCellMarginLeft = 4;
	private int headerCellMarginLeftOrder = 17;

	private int bodyRowAlignments = 1;
	private int bodyRowHeight = 24;
	private int bodyLineHeight = 20;
	private int bodyCellMarginTop = 2;
	private int bodyCellMarginRight = 4;
	private int bodyCellMarginBottom = 2;
	private int bodyCellMarginLeft = 4;

	private int pageResizingActionDelay = 500;
	private int scrollbarActionDelay = 600;
	private int actionTrayDelay = 300;
	private int loadingMessageShowDelay = 100;
	private int loadingMessageHideDelay = 300;
	private float loadingMessageOpacity = 0.5f;

	private List<Integer> columnsFormatted = new ArrayList<Integer>();
	private List<String> columnsFreezeEnabled = new ArrayList<String>();
	private List<String> columnsUnfreezeEnabled = new ArrayList<String>();
	private List<String> columnsHideEnabled = new ArrayList<String>();

	private boolean enableAllHeaderColumns = false;
	private boolean enableFixedTableHeight = false;
	private boolean enableAllColumns = false;
	private boolean enableAllRows = false;

	private boolean enableQuickSearch = false;
	private boolean enableQuickSearchImmediate = false;
	private boolean enableQuickSearchVisibleAtStartup = false;
	private boolean enableQuickSearchHide = false;

	private boolean enableSearch = false;
	private boolean enableSearchFromatting = false;
	private boolean enableSearchImmediate = false;
	private boolean enableSearchVisibleAtStartup = false;
	private boolean enableSearchHide = false;

	private boolean enablePageableTable = false;
	private boolean enableFrozenTable = false;
	private boolean enableFrozenTableVisible = false;
	private boolean enableHideFrozenTable = false;
	private boolean enableResizeFrozenTable = false;
	private boolean enableEmptyFrozenTable = false;
	private boolean enableFreezeUnfreezeColumns = false;
	private boolean enableFreezeAnyColumns = false;
	private boolean enableUnfreezeAnyColumns = false;
	private boolean enableHideUnhideColumns = false;
	private boolean enableHideAnyColumns = false;
	private boolean enableHighlightActiveRow = false;

	private boolean enableOrderedFrozenTable = false;
	private boolean enableOrderedTable = false;

	private boolean enableFilterInfo = false;
	private boolean enableFilteredFrozenTable = false;
	private boolean enableFilteredTable = false;
	private boolean enableContextMenuForFilteredFrozenTable = false;
	private boolean enableContextMenuForFilteredTable = false;

	private boolean enableExpandedFrozenHeaderCell = false;
	private boolean enableExpandedFrozenHeaderCellAlwaysVisible = false;
	private boolean enableExpandedFrozenHeaderCellForActiveColumn = false;
	private boolean enableExpandedFrozenBodyCell = false;
	private boolean enableExpandedFrozenBodyCellAlwaysVisible = false;

	private boolean enableExpandedHeaderCell = false;
	private boolean enableExpandedHeaderCellAlwaysVisible = false;
	private boolean enableExpandedHeaderCellForActiveColumn = false;
	private boolean enableExpandedBodyCell = false;
	private boolean enableExpandedBodyCellAlwaysVisible = false;

	private boolean enableWordWrapInHeaderCells = false;
	private boolean enableWordWrapInBodyCells = false;

	private boolean enableOddAndEvenColoringRows = false;
	private boolean enableHorizontalLines = false;
	private boolean enableHeaderVerticalLines = true;
	private boolean enableBodyVerticalLines = true;
	private boolean enableLoadingMessage = false;

	private boolean enableTooltip = false;
	private boolean enableWordWrapInTooltip = false;
	private String tooltipPosition = null;
	private int tooltipMinimumWidth = 0;
	private int tooltipMaximumWidth = 0;

	public void process(ValueMap config) {
		if (config == null || config.getKeySet().size() == 0) {
			return;
		}

		if (config.containsKey("enableFrozenTable")) {
			enableFrozenTable = config.getBoolean("enableFrozenTable");
		}

		if (config.containsKey("enableFrozenTableVisible")) {
			enableFrozenTableVisible = enableFrozenTable
					&& config.getBoolean("enableFrozenTableVisible");
		}

		if (config.containsKey("enableHideFrozenTable")) {
			enableHideFrozenTable = enableFrozenTable
					&& config.getBoolean("enableHideFrozenTable");
		}

		if (config.containsKey("enableResizeFrozenTable")) {
			enableResizeFrozenTable = enableFrozenTable
					&& config.getBoolean("enableResizeFrozenTable");
		}

		if (config.containsKey("enableEmptyFrozenTable")) {
			enableEmptyFrozenTable = enableFrozenTable
					&& config.getBoolean("enableEmptyFrozenTable");
		}

		if (config.containsKey("enablePageableTable")) {
			enablePageableTable = config.getBoolean("enablePageableTable");
		}

		if (config.containsKey("enableAllHeaderColumns")) {
			enableAllHeaderColumns = config
					.getBoolean("enableAllHeaderColumns");
		}

		if (config.containsKey("enableFixedTableHeight")) {
			enableFixedTableHeight = config
					.getBoolean("enableFixedTableHeight");
		}

		if (config.containsKey("enableAllRows") && !enablePageableTable) {
			enableAllRows = config.getBoolean("enableAllRows");
		}

		if (config.containsKey("enableAllColumns")) {
			enableAllColumns = config.getBoolean("enableAllColumns");
		}

		if (config.containsKey("enableColumnNumberInWaitBox")) {
			// table.showColumnNumbersInWaitBox(config
			// .getBoolean("enableColumnNumberInWaitBox"));
		}

		if (config.containsKey("enableOrderedFrozenTable")) {
			enableOrderedFrozenTable = enableFrozenTable
					&& config.getBoolean("enableOrderedFrozenTable");
		}

		if (config.containsKey("enableOrderedTable")) {
			enableOrderedTable = config.getBoolean("enableOrderedTable");
		}

		if (config.containsKey("enableFilteredFrozenTable")) {
			enableFilteredFrozenTable = enableFrozenTable
					&& config.getBoolean("enableFilteredFrozenTable");
		}

		if (config.containsKey("enableFilteredTable")) {
			enableFilteredTable = config.getBoolean("enableFilteredTable");
		}

		if (config.containsKey("enableContextMenuForFilteredFrozenTable")) {
			enableContextMenuForFilteredFrozenTable = enableFrozenTable
					&& enableFilteredFrozenTable
					&& config
							.getBoolean("enableContextMenuForFilteredFrozenTable");
		}

		if (config.containsKey("enableContextMenuForFilteredTable")) {
			enableContextMenuForFilteredTable = enableFilteredTable
					&& config.getBoolean("enableContextMenuForFilteredTable");
		}

		if (config.containsKey("enableFilterInfo")) {
			enableFilterInfo = (enableFilteredFrozenTable || enableFilteredTable)
					&& config.getBoolean("enableFilterInfo");
		}

		if (config.containsKey("enableQuickSearch")) {
			enableQuickSearch = (enableFilteredFrozenTable || enableFilteredTable)
					&& config.getBoolean("enableQuickSearch");
		}

		if (config.containsKey("enableQuickSearchImmediate")) {
			enableQuickSearchImmediate = enableQuickSearch
					&& config.getBoolean("enableQuickSearchImmediate");
		}

		if (config.containsKey("enableSearchsVisibleAtStartup")) {
			enableQuickSearchVisibleAtStartup = enableQuickSearch
					&& (config.getBoolean("enableQuickSearchVisibleAtStartup") || !config
							.getBoolean("enableQuickSearchHide"));
		}

		if (config.containsKey("enableQuickSearchHide")) {
			enableQuickSearchHide = enableQuickSearch
					&& config.getBoolean("enableQuickSearchHide");
		}

		if (config.containsKey("enableSearch")) {
			enableSearch = (enableFilteredFrozenTable || enableFilteredTable)
					&& config.getBoolean("enableSearch");
		}

		if (config.containsKey("enableSearchFromatting")) {
			enableSearchFromatting = enableSearch
					&& config.getBoolean("enableSearchFromatting");
		}

		if (config.containsKey("enableSearchImmediate")) {
			enableSearchImmediate = enableSearch
					&& config.getBoolean("enableSearchImmediate");
		}

		if (config.containsKey("enableSearchVisibleAtStartup")) {
			enableSearchVisibleAtStartup = enableSearch
					&& (config.getBoolean("enableSearchVisibleAtStartup") || !config
							.getBoolean("enableSearchHide"));
		}

		if (config.containsKey("enableSearchHide")) {
			enableSearchHide = enableSearch
					&& config.getBoolean("enableSearchHide");
		}

		if (config.containsKey("enableFreezeUnfreezeColumns")) {
			enableFreezeUnfreezeColumns = config
					.getBoolean("enableFreezeUnfreezeColumns");
		}

		if (config.containsKey("enableFreezeAnyColumns")) {
			enableFreezeAnyColumns = enableFrozenTable
					&& config.getBoolean("enableFreezeAnyColumns");
		}

		if (config.containsKey("enableUnfreezeAnyColumns")) {
			enableUnfreezeAnyColumns = enableFrozenTable
					&& config.getBoolean("enableUnfreezeAnyColumns");
		}

		if (config.containsKey("enableHideUnhideColumns")) {
			enableHideUnhideColumns = config
					.getBoolean("enableHideUnhideColumns");
		}

		if (config.containsKey("enableHideAnyColumns")) {
			enableHideAnyColumns = config.getBoolean("enableHideAnyColumns");
		}

		if (config.containsKey("enableHighlightActiveRow")) {
			enableHighlightActiveRow = config
					.getBoolean("enableHighlightActiveRow");
		}

		if (config.containsKey("enableOddAndEvenColoringRows")) {
			enableOddAndEvenColoringRows = config
					.getBoolean("enableOddAndEvenColoringRows");
		}

		if (config.containsKey("enableWordWrapInHeaderCells")) {
			enableWordWrapInHeaderCells = config
					.getBoolean("enableWordWrapInHeaderCells");
		}

		if (config.containsKey("enableWordWrapInBodyCells")) {
			enableWordWrapInBodyCells = config
					.getBoolean("enableWordWrapInBodyCells");
		}

		if (config.containsKey("enableExpandedFrozenHeaderCell")) {
			enableExpandedFrozenHeaderCell = enableFrozenTable
					&& config.getBoolean("enableExpandedFrozenHeaderCell");
			if (enableExpandedFrozenHeaderCell
					&& config
							.containsKey("enableExpandedFrozenHeaderCellAlwaysVisible")) {
				enableExpandedFrozenHeaderCellAlwaysVisible = config
						.getBoolean("enableExpandedFrozenHeaderCellAlwaysVisible");
			}
		}

		if (config.containsKey("enableExpandedFrozenHeaderCellForActiveColumn")) {
			enableExpandedFrozenHeaderCellForActiveColumn = enableFrozenTable
					&& config
							.getBoolean("enableExpandedFrozenHeaderCellForActiveColumn");
		}

		if (config.containsKey("enableExpandedFrozenBodyCell")) {
			enableExpandedFrozenBodyCell = enableFrozenTable
					&& config.getBoolean("enableExpandedFrozenBodyCell");
			if (enableExpandedFrozenBodyCell
					&& config
							.containsKey("enableExpandedFrozenBodyCellAlwaysVisible")) {
				enableExpandedFrozenBodyCellAlwaysVisible = config
						.getBoolean("enableExpandedFrozenBodyCellAlwaysVisible");
			}
		}

		if (config.containsKey("enableExpandedHeaderCell")) {
			enableExpandedHeaderCell = config
					.getBoolean("enableExpandedHeaderCell");
			if (enableExpandedHeaderCell
					&& config
							.containsKey("enableExpandedHeaderCellAlwaysVisible")) {
				enableExpandedHeaderCellAlwaysVisible = config
						.getBoolean("enableExpandedHeaderCellAlwaysVisible");
			}
		}

		if (config.containsKey("enableExpandedHeaderCellForActiveColumn")) {
			enableExpandedHeaderCellForActiveColumn = config
					.getBoolean("enableExpandedHeaderCellForActiveColumn");
		}

		if (config.containsKey("enableExpandedBodyCell")) {
			enableExpandedBodyCell = config
					.getBoolean("enableExpandedBodyCell");
			if (enableExpandedBodyCell
					&& config
							.containsKey("enableExpandedBodyCellAlwaysVisible")) {
				enableExpandedBodyCellAlwaysVisible = config
						.getBoolean("enableExpandedBodyCellAlwaysVisible");
			}
		}

		if (config.containsKey("enableHorizontalLines")) {
			enableHorizontalLines = config.getBoolean("enableHorizontalLines");
		}

		if (config.containsKey("enableHeaderVerticalLines")) {
			enableHeaderVerticalLines = config
					.getBoolean("enableHeaderVerticalLines");
		}

		if (config.containsKey("enableBodyVerticalLines")) {
			enableBodyVerticalLines = config
					.getBoolean("enableBodyVerticalLines");
		}

		if (config.containsKey("enableLoadingMessage")) {
			enableLoadingMessage = config.getBoolean("enableLoadingMessage");
		}

		if (config.containsKey("pageResizingActionDelay")) {
			pageResizingActionDelay = config.getInt("pageResizingActionDelay");
		}

		if (config.containsKey("scrollbarActionDelay")) {
			scrollbarActionDelay = config.getInt("scrollbarActionDelay");
		}

		if (config.containsKey("actionTrayDelay")) {
			actionTrayDelay = config.getInt("actionTrayDelay");
		}

		if (config.containsKey("loadingMessageOpacity")) {
			loadingMessageOpacity = Float.parseFloat(config
					.getString("loadingMessageOpacity"));
		}

		if (config.containsKey("loadingMessageShowDelay")
				&& config.containsKey("loadingMessageHideDelay")) {
			loadingMessageShowDelay = config.getInt("loadingMessageShowDelay");
			loadingMessageHideDelay = config.getInt("loadingMessageHideDelay");
		}

		if (config.containsKey("tableFrozenMinimalWidth") && enableFrozenTable) {
			tableFrozenMinimalWidth = config.getInt("tableFrozenMinimalWidth");
		}

		if (config.containsKey("contextMenuDepthForFilteredFrozenTable")) {
			contextMenuDepthForFilteredFrozenTable = config
					.getInt("contextMenuDepthForFilteredFrozenTable");
		}

		if (config.containsKey("contextMenuDepthForFilteredTable")) {
			contextMenuDepthForFilteredTable = config
					.getInt("contextMenuDepthForFilteredTable");
		}

		if (config.containsKey("pagerPages")) {
			pagerPages = config.getInt("pagerPages");
		}

		if (config.containsKey("pageSize")) {
			pageSize = config.getInt("pageSize");
		}

		if (config.containsKey("tableToolsPosition")) {
			tableToolsPosition = "-" + config.getString("tableToolsPosition");
		}

		if (config.containsKey("filterInfoPosition")) {
			filterInfoPosition = config.getString("filterInfoPosition");
		}

		if (config.containsKey("tableMinimalWidth")) {
			tableMinimalWidth = config.getInt("tableMinimalWidth");
		}

		if (config.containsKey("headerRowsAlignment")) {
			headerRowsAlignment = config.getInt("headerRowsAlignment");
		}

		if (config.containsKey("headerCellMarginTop")) {
			headerCellMarginTop = config.getInt("headerCellMarginTop");
		}

		if (config.containsKey("headerCellMarginRight")) {
			headerCellMarginRight = config.getInt("headerCellMarginRight");
		}

		if (config.containsKey("headerCellMarginBottom")) {
			headerCellMarginBottom = config.getInt("headerCellMarginBottom");
		}

		if (config.containsKey("headerCellMarginLeft")) {
			headerCellMarginLeft = config.getInt("headerCellMarginLeft");
		}

		if (config.containsKey("headerLineHeight")) {
			headerLineHeight = config.getInt("headerLineHeight");
		}

		// don't move
		if (config.containsKey("headerRowHeight")) {
			headerRowHeight = Math.max(config.getInt("headerRowHeight"),
					headerLineHeight + headerCellMarginTop
							+ headerCellMarginBottom + 1);
		}

		if (config.containsKey("bodyRowsAlignment")) {
			bodyRowAlignments = config.getInt("bodyRowsAlignment");
		}

		if (config.containsKey("bodyCellMarginTop")) {
			bodyCellMarginTop = config.getInt("bodyCellMarginTop");
		}

		if (config.containsKey("bodyCellMarginRight")) {
			bodyCellMarginRight = config.getInt("bodyCellMarginRight");
		}

		if (config.containsKey("bodyCellMarginBottom")) {
			bodyCellMarginBottom = config.getInt("bodyCellMarginBottom");
		}

		if (config.containsKey("bodyCellMarginLeft")) {
			bodyCellMarginLeft = config.getInt("bodyCellMarginLeft");
		}

		if (config.containsKey("bodyLineHeight")) {
			bodyLineHeight = config.getInt("bodyLineHeight");
		}

		if (config.containsKey("enableTooltip")) {
			enableTooltip = config.getBoolean("enableTooltip");
		}

		if (config.containsKey("enableWordWrapInTooltip")) {
			enableWordWrapInTooltip = config
					.getBoolean("enableWordWrapInTooltip");
		}

		if (config.containsKey("tooltipPosition")) {
			tooltipPosition = config.getString("tooltipPosition");
		}

		if (config.containsKey("tooltipMinimumWidth")) {
			tooltipMinimumWidth = Math.max(0,
					config.getInt("tooltipMinimumWidth"));
		}

		if (config.containsKey("tooltipMaximumWidth")) {
			tooltipMaximumWidth = Math.max(0,
					config.getInt("tooltipMaximumWidth"));
		}

		// don't move
		if (config.containsKey("bodyRowHeight")) {
			bodyRowHeight = Math.max(config.getInt("bodyRowHeight"),
					bodyLineHeight + bodyCellMarginTop + bodyCellMarginBottom
							+ 1);
		}

		// don't move
		if (config.containsKey("tableHeight")) {
			tableHeight = enablePageableTable ? bodyRowHeight * pageSize
					: config.getInt("tableHeight") - headerRowHeight;
		}
	}

	public int getHeaderRowsAlignment() {
		return headerRowsAlignment;
	}

	public int getTableFrozenMinimalWidth() {
		return tableFrozenMinimalWidth;
	}

	public int getTableMinimalWidth() {
		return tableMinimalWidth;
	}

	public int getContextMenuDepthForFilteredFrozenTable() {
		return contextMenuDepthForFilteredFrozenTable;
	}

	public int getContextMenuDepthForFilteredTable() {
		return contextMenuDepthForFilteredTable;
	}

	public int getTableHeight() {
		return tableHeight;
	}

	public int getPagerPages() {
		return pagerPages;
	}

	public int getPageSize() {
		return pageSize;
	}

	public String getTableToolsPosition() {
		return tableToolsPosition;
	}

	public boolean isFilterInfoOnTop() {
		return "top".equals(filterInfoPosition);
	}

	public boolean isFilterInfoOnBottom() {
		return "bottom".equals(filterInfoPosition);
	}

	public int getHeaderRowHeight() {
		return headerRowHeight;
	}

	public int getHeaderLineHeight() {
		return headerLineHeight;
	}

	public int getHeaderCellMarginTop() {
		return headerCellMarginTop;
	}

	public int getHeaderCellMarginRight() {
		return headerCellMarginRight;
	}

	public int getHeaderCellMarginBottom() {
		return headerCellMarginBottom;
	}

	public int getHeaderCellMarginLeft() {
		return headerCellMarginLeft;
	}

	public int getHeaderCellMarginLeftOrdered() {
		return headerCellMarginLeftOrder;
	}

	public int getPageResizingActionDelay() {
		return pageResizingActionDelay;
	}

	public int getScrollbarActionDelay() {
		return scrollbarActionDelay;
	}

	public int getLoadingMessageShowDelay() {
		return loadingMessageShowDelay;
	}

	public int getLoadingMessageHideDelay() {
		return loadingMessageHideDelay;
	}

	public float getLoadingMessageOpacity() {
		return loadingMessageOpacity;
	}

	public int getBodyRowAlignments() {
		return bodyRowAlignments;
	}

	public int getBodyRowHeight() {
		return bodyRowHeight;
	}

	public int getBodyLineHeight() {
		return bodyLineHeight;
	}

	public int getBodyCellMarginTop() {
		return bodyCellMarginTop;
	}

	public int getBodyCellMarginRight() {
		return bodyCellMarginRight;
	}

	public int getBodyCellMarginBottom() {
		return bodyCellMarginBottom;
	}

	public int getBodyCellMarginLeft() {
		return bodyCellMarginLeft;
	}

	public int getActionTrayDelay() {
		return actionTrayDelay;
	}

	public boolean isColumnFormatted(int column) {
		return columnsFormatted.contains(column);
	}

	public void setColumnsFormatted(int[] columns) {
		if (columns != null && columns.length > 0) {
			columnsFormatted.clear();
			for (int column : columns) {
				columnsFormatted.add(column);
			}
		}
	}

	public boolean isColumnFreezeEnabled(int column) {
		return columnsFreezeEnabled.contains("+" + column);
	}

	public boolean isColumnFreezeDisabled(int column) {
		return columnsFreezeEnabled.contains("-" + column);
	}

	public void enableColumnsFreeze(String[] columns) {
		if (columns != null && columns.length > 0) {
			columnsFreezeEnabled.clear();
			for (String column : columns) {
				columnsFreezeEnabled.add(column);
			}
		}
	}

	public boolean isColumnUnfreezeEnabled(int column) {
		return columnsUnfreezeEnabled.contains("+" + column);
	}

	public boolean isColumnUnfreezeDisabled(int column) {
		return columnsUnfreezeEnabled.contains("-" + column);
	}

	public void enableColumnsUnfreeze(String[] columns) {
		if (columns != null && columns.length > 0) {
			columnsUnfreezeEnabled.clear();
			for (String column : columns) {
				columnsUnfreezeEnabled.add(column);
			}
		}
	}

	public boolean isColumnHideEnabled(int column) {
		if (columnsHideEnabled.contains("-" + column)) {
			return false;
		} else if (columnsHideEnabled.contains("+" + column)) {
			return true;
		}
		return enableHideAnyColumns;
	}

	public void enableColumnsHide(String[] columns) {
		if (columns != null && columns.length > 0) {
			columnsHideEnabled.clear();
			for (String column : columns) {
				columnsHideEnabled.add(column);
			}
		}
	}

	public boolean isEnabledAllHeaderColumns() {
		return enableAllHeaderColumns;
	}

	public boolean isEnabledFixedTableHeight() {
		return enableFixedTableHeight;
	}

	public boolean isEnabledAllColumns() {
		return enableAllColumns;
	}

	public boolean isEnabledAllRows() {
		return enableAllRows;
	}

	public boolean isEnabledQuickSearch() {
		return enableQuickSearch;
	}

	public boolean isEnabledQuickSearchImmediate() {
		return enableQuickSearchImmediate;
	}

	public boolean isEnabledQuickSearchVisibleAtStartup() {
		return enableQuickSearchVisibleAtStartup;
	}

	public boolean isEnabledQuickSearchHide() {
		return enableQuickSearchHide;
	}

	public boolean isEnabledSearch() {
		return enableSearch;
	}

	public boolean isEnabledSearchFromatting() {
		return enableSearchFromatting;
	}

	public boolean isEnabledSearchImmediate() {
		return enableSearchImmediate;
	}

	public boolean isEnabledSearchVisibleAtStartup() {
		return enableSearchVisibleAtStartup;
	}

	public boolean isEnabledSearchHide() {
		return enableSearchHide;
	}

	public boolean isEnabledPageableTable() {
		return enablePageableTable;
	}

	public boolean isEnabledFrozenTable() {
		return enableFrozenTable;
	}

	public boolean isEnabledFrozenTableVisible() {
		return enableFrozenTableVisible;
	}

	public boolean isEnabledHideFrozenTable() {
		return enableHideFrozenTable;
	}

	public boolean isEnabledResizeFrozenTable() {
		return enableResizeFrozenTable;
	}

	public boolean isEnabledEmptyFrozenTable() {
		return enableEmptyFrozenTable;
	}

	public boolean isEnabledFreezeUnfreezeColumns() {
		return enableFreezeUnfreezeColumns;
	}

	public boolean isEnabledFreezeAnyColumns() {
		return enableFreezeAnyColumns;
	}

	public boolean isEnabledUnfreezeAnyColumns() {
		return enableUnfreezeAnyColumns;
	}

	public boolean isEnabledHideUnhideColumns() {
		return enableHideUnhideColumns;
	}

	public boolean isEnabledHideAnyColumns() {
		return enableHideAnyColumns;
	}

	public boolean isEnabledFreeze(int column) {
		if (isColumnFreezeEnabled(column)) {
			return true;
		} else if (isColumnFreezeDisabled(column)) {
			return false;
		}
		return isEnabledFreezeAnyColumns();
	}

	public boolean isEnabledUnfreeze(int column) {
		if (isColumnUnfreezeEnabled(column)) {
			return true;
		} else if (isColumnUnfreezeDisabled(column)) {
			return false;
		}
		return isEnabledUnfreezeAnyColumns();
	}

	public boolean isEnabledHighlightActiveRow() {
		return enableHighlightActiveRow;
	}

	public boolean isEnabledOrderedFrozenTable() {
		return enableOrderedFrozenTable;
	}

	public boolean isEnabledOrderedTable() {
		return enableOrderedTable;
	}

	public boolean isEnabledFilteredFrozenTable() {
		return enableFilteredFrozenTable;
	}

	public boolean isEnabledFilteredTable() {
		return enableFilteredTable;
	}

	public boolean isEnabledContextMenuForFilteredFrozenTable() {
		return enableContextMenuForFilteredFrozenTable;
	}

	public boolean isEnabledContextMenuForFilteredTable() {
		return enableContextMenuForFilteredTable;
	}

	public boolean isEnabledExpandedFrozenHeaderCell() {
		return enableExpandedFrozenHeaderCell;
	}

	public boolean isEnabledExpandedFrozenHeaderCellAlwaysVisible() {
		return enableExpandedFrozenHeaderCellAlwaysVisible;
	}

	public boolean isEnabledExpandedFrozenHeaderCellForActiveColumn() {
		return enableExpandedFrozenHeaderCellForActiveColumn;
	}

	public boolean isEnabledExpandedFrozenBodyCell() {
		return enableExpandedFrozenBodyCell;
	}

	public boolean isEnabledExpandedFrozenBodyCellAlwaysVisible() {
		return enableExpandedFrozenBodyCellAlwaysVisible;
	}

	public boolean isEnabledExpandedHeaderCell() {
		return enableExpandedHeaderCell;
	}

	public boolean isEnabledExpandedHeaderCellAlwaysVisible() {
		return enableExpandedHeaderCellAlwaysVisible;
	}

	public boolean isEnabledExpandedHeaderCellForActiveColumn() {
		return enableExpandedHeaderCellForActiveColumn;
	}

	public boolean isEnabledExpandedBodyCell() {
		return enableExpandedBodyCell;
	}

	public boolean isEnabledExpandedBodyCellAlwaysVisible() {
		return enableExpandedBodyCellAlwaysVisible;
	}

	public boolean isEnabledWordWrapInHeaderCells() {
		return enableWordWrapInHeaderCells;
	}

	public boolean isEnabledWordWrapInBodyCells() {
		return enableWordWrapInBodyCells;
	}

	public boolean isEnabledFilterInfo() {
		return enableFilterInfo;
	}

	public boolean isEnabledOddAndEvenColoringRows() {
		return enableOddAndEvenColoringRows;
	}

	public boolean isEnabledHorizontalLines() {
		return enableHorizontalLines;
	}

	public boolean isEnabledHeaderVerticalLines() {
		return enableHeaderVerticalLines;
	}

	public boolean isEnabledBodyVerticalLines() {
		return enableBodyVerticalLines;
	}

	public boolean isEnabledLoadingMessage() {
		return enableLoadingMessage;
	}

	public boolean isEnabledTooltip() {
		return enableTooltip;
	}

	public boolean isEnabledWordWrapInTooltip() {
		return enableWordWrapInTooltip;
	}

	public String getTooltipPosition() {
		return tooltipPosition;
	}

	public int getTooltipMinimumWidth() {
		return tooltipMinimumWidth;
	}

	public int getTooltipMaximumWidth() {
		return tooltipMaximumWidth;
	}

}
