/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client;

import java.util.HashMap;
import java.util.Map;

import com.vaadin.terminal.gwt.client.ValueMap;

public class Translation {

	public static final String ACTION = "ACTION";
	public static final String ASCENDING_ORDER_FOR_COLUMN = "ASCENDING_ORDER_FOR_COLUMN";
	public static final String CSV_FILE = "CSV_FILE";
	public static final String DATA_ERROR = "DATA_ERROR";
	public static final String DESCENDING_ORDER_FOR_COLUMN = "DESCENDING_ORDER_FOR_COLUMN";
	public static final String EXPORT = "EXPORT";
	public static final String ERROR = "ERROR";
	public static final String FILTERS = "FILTERS";
	public static final String FREEZE_COLUMN = "FREEZE_COLUMN";
	public static final String FREEZE_UNFREEZE_COLUMNS = "FREEZE_UNFREEZE_COLUMNS";
	public static final String HIDE_COLUMN = "HIDE_COLUMN";
	public static final String HIDE_UNHIDE_COLUMNS = "HIDE_UNHIDE_COLUMNS";
	public static final String IS_EMPTY = "IS_EMPTY";
	public static final String IS_NOT_EMPTY = "IS_NOT_EMPTY";
	public static final String LOADING_DATA_PLEASE_WAIT = "LOADING_DATA_PLEASE_WAIT";
	public static final String MS_ACCESS_FILE = "MS_ACCESS_FILE";
	public static final String MS_EXCEL_FILE = "MS_EXCEL_FILE";
	public static final String NO_DATA_TO_DISPLAY = "NO_DATA_TO_DISPLAY";
	public static final String NO_NEW_DATA = "NO_NEW_DATA";
	public static final String QUICK_SEARCH = "QUICK_SEARCH";
	public static final String PDF_FILE = "PDF_FILE";
	public static final String REMOVE_ALL_ORDERS = "REMOVE_ALL_ORDERS";
	public static final String REMOVE_FILTERS_FROM_COLUMN = "REMOVE_FILTERS_FROM_COLUMN";
	public static final String REMOVE_ORDER_FOR_COLUMN = "REMOVE_ORDER_FOR_COLUMN";
	public static final String SEARCH = "SEARCH";
	public static final String SEARCH_IN_COLUMNS = "SEARCH_IN_COLUMNS";
	public static final String SIZE_OF_THE_REQUESTED_DATA_IS_ZERO = "SIZE_OF_THE_REQUESTED_DATA_IS_ZERO";
	public static final String SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS = "SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS";
	public static final String UNFREEZE_COLUMN = "UNFREEZE_COLUMN";
	public static final String WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED = "WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED";

	private Map<String, String> translatedWords = new HashMap<String, String>();

	public String get(String translatedWord) {
		if (translatedWords.containsKey(translatedWord)) {
			return translatedWords.get(translatedWord);
		}
		return null;
	}

	public void set(Map<String, String> translatedWords) {
		this.translatedWords.putAll(translatedWords);
	}

	public void process(ValueMap translatedWords) {
		if (translatedWords == null || translatedWords.getKeySet().size() == 0) {
			return;
		}

		if (translatedWords.containsKey(ACTION)) {
			this.translatedWords.put(ACTION, translatedWords.getString(ACTION));
		}

		if (translatedWords.containsKey(ASCENDING_ORDER_FOR_COLUMN)) {
			this.translatedWords.put(ASCENDING_ORDER_FOR_COLUMN,
					translatedWords.getString(ASCENDING_ORDER_FOR_COLUMN)
							+ " '_'");
		}

		if (translatedWords.containsKey(DATA_ERROR)) {
			this.translatedWords.put(DATA_ERROR,
					translatedWords.getString(DATA_ERROR));
		}

		if (translatedWords.containsKey(DESCENDING_ORDER_FOR_COLUMN)) {
			this.translatedWords.put(DESCENDING_ORDER_FOR_COLUMN,
					translatedWords.getString(DESCENDING_ORDER_FOR_COLUMN)
							+ " '_'");
		}

		if (translatedWords.containsKey(EXPORT)) {
			this.translatedWords.put(EXPORT, translatedWords.getString(EXPORT));
		}

		if (translatedWords.containsKey(ERROR)) {
			this.translatedWords.put(ERROR, translatedWords.getString(ERROR));
		}

		if (translatedWords.containsKey(CSV_FILE)) {
			this.translatedWords.put(CSV_FILE,
					translatedWords.getString(CSV_FILE));
		}

		if (translatedWords.containsKey(MS_ACCESS_FILE)) {
			this.translatedWords.put(MS_ACCESS_FILE,
					translatedWords.getString(MS_ACCESS_FILE));
		}

		if (translatedWords.containsKey(PDF_FILE)) {
			this.translatedWords.put(PDF_FILE,
					translatedWords.getString(PDF_FILE));
		}

		if (translatedWords.containsKey(MS_EXCEL_FILE)) {
			this.translatedWords.put(MS_EXCEL_FILE,
					translatedWords.getString(MS_EXCEL_FILE));
		}

		if (translatedWords.containsKey(FILTERS)) {
			this.translatedWords.put(FILTERS,
					translatedWords.getString(FILTERS));
		}

		if (translatedWords.containsKey(FREEZE_COLUMN)) {
			this.translatedWords.put(FREEZE_COLUMN,
					translatedWords.getString(FREEZE_COLUMN) + " '_'");
		}

		if (translatedWords.containsKey(FREEZE_UNFREEZE_COLUMNS)) {
			this.translatedWords.put(FREEZE_UNFREEZE_COLUMNS,
					translatedWords.getString(FREEZE_UNFREEZE_COLUMNS));
		}

		if (translatedWords.containsKey(HIDE_COLUMN)) {
			this.translatedWords.put(HIDE_COLUMN,
					translatedWords.getString(HIDE_COLUMN) + " '_'");
		}

		if (translatedWords.containsKey(HIDE_UNHIDE_COLUMNS)) {
			this.translatedWords.put(HIDE_UNHIDE_COLUMNS,
					translatedWords.getString(HIDE_UNHIDE_COLUMNS));
		}

		if (translatedWords.containsKey(IS_EMPTY)) {
			this.translatedWords.put(IS_EMPTY,
					translatedWords.getString(IS_EMPTY));
		}

		if (translatedWords.containsKey(IS_NOT_EMPTY)) {
			this.translatedWords.put(IS_NOT_EMPTY,
					translatedWords.getString(IS_NOT_EMPTY));
		}

		if (translatedWords.containsKey(LOADING_DATA_PLEASE_WAIT)) {
			this.translatedWords.put(LOADING_DATA_PLEASE_WAIT,
					translatedWords.getString(LOADING_DATA_PLEASE_WAIT));
		}

		if (translatedWords.containsKey(NO_DATA_TO_DISPLAY)) {
			this.translatedWords.put(NO_DATA_TO_DISPLAY,
					translatedWords.getString(NO_DATA_TO_DISPLAY));
		}

		if (translatedWords.containsKey(NO_NEW_DATA)) {
			this.translatedWords.put(NO_NEW_DATA,
					translatedWords.getString(NO_NEW_DATA));
		}

		if (translatedWords.containsKey(QUICK_SEARCH)) {
			this.translatedWords.put(QUICK_SEARCH,
					translatedWords.getString(QUICK_SEARCH));
		}

		if (translatedWords.containsKey(REMOVE_ALL_ORDERS)) {
			this.translatedWords.put(REMOVE_ALL_ORDERS,
					translatedWords.getString(REMOVE_ALL_ORDERS));
		}

		if (translatedWords.containsKey(REMOVE_FILTERS_FROM_COLUMN)) {
			this.translatedWords.put(REMOVE_FILTERS_FROM_COLUMN,
					translatedWords.getString(REMOVE_FILTERS_FROM_COLUMN)
							+ " '_'");
		}

		if (translatedWords.containsKey(REMOVE_ORDER_FOR_COLUMN)) {
			this.translatedWords.put(REMOVE_ORDER_FOR_COLUMN,
					translatedWords.getString(REMOVE_ORDER_FOR_COLUMN)
							+ " '_'");
		}

		if (translatedWords.containsKey(SEARCH)) {
			this.translatedWords.put(SEARCH, translatedWords.getString(SEARCH));
		}

		if (translatedWords.containsKey(SEARCH_IN_COLUMNS)) {
			this.translatedWords.put(SEARCH_IN_COLUMNS,
					translatedWords.getString(SEARCH_IN_COLUMNS));
		}

		if (translatedWords.containsKey(SIZE_OF_THE_REQUESTED_DATA_IS_ZERO)) {
			this.translatedWords.put(SIZE_OF_THE_REQUESTED_DATA_IS_ZERO,
					translatedWords
							.getString(SIZE_OF_THE_REQUESTED_DATA_IS_ZERO));
		}

		if (translatedWords
				.containsKey(SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS)) {
			this.translatedWords
					.put(SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS,
							translatedWords
									.getString(SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS));
		}

		if (translatedWords.containsKey(UNFREEZE_COLUMN)) {
			this.translatedWords.put(UNFREEZE_COLUMN,
					translatedWords.getString(UNFREEZE_COLUMN) + " '_'");
		}

		if (translatedWords
				.containsKey(WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED)) {
			this.translatedWords
					.put(WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED,
							translatedWords
									.getString(WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED));
		}
	}

}
