/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.GQuery;
import com.vaadin.terminal.gwt.client.UIDL;

public class Filter extends TableComponent {

	private String mainId = "#v-dtable";
	private String frozenBodyId = "table#dt-frozen-body-table";
	private String bodyId = "table#dt-body-table";
	private String filterHtml = new String();

	private Map<Integer, List<Object>> columnsFilter = new HashMap<Integer, List<Object>>();

	public Filter(VDTable dTable) {
		super(dTable);
	}

	private void setFilter(String[] filters) {
		if (filters != null && filters.length > 0) {
			Map<Integer, List<Object>> newFilters = new HashMap<Integer, List<Object>>();
			for (int filter = 0; filter < filters.length; filter += 3) {
				int column = Integer.parseInt(filters[filter]);
				List<Object> tempFilters = newFilters.get(column);
				if (tempFilters == null) {
					tempFilters = new ArrayList<Object>();
				}
				tempFilters.add(new String[] { filters[filter + 1],
						filters[filter + 2] });
				newFilters.put(column, tempFilters);
			}
			if (!newFilters.isEmpty()) {
				columnsFilter = new HashMap<Integer, List<Object>>(newFilters);
			}
		}
	}

	private void setFilterHtml(String filterHtml) {
		this.filterHtml = filterHtml;
	}

	private void showFilterHtml() {
		$(mainId + " div#dt-filter-info").remove();
		if (getDTable().config().isEnabledFilterInfo() && !filterHtml.isEmpty()) {
			filterHtml = "<b><u>"
					+ getDTable().translation().get(Translation.FILTERS)
					+ "</u></b> " + filterHtml;

			GQuery mainContainerWrapper = $(mainId).find(
					"div#dt-main-container-wrapper");
			GQuery tableTools = $(mainId).find(
					"div#dt-table-tools" + getDTable().getId());
			if (getDTable().config().isFilterInfoOnTop()) {
				if (getDTable().tools().isVisible()
						&& getDTable().tools().isOnTop()) {
					tableTools.before($("<div/>").id("dt-filter-info")
							.addClass("dt-filter-info")
							.html(filterHtml.toString()));
				} else {
					mainContainerWrapper.before($("<div/>")
							.id("dt-filter-info").addClass("dt-filter-info")
							.html(filterHtml.toString()));
				}
			} else if (getDTable().config().isFilterInfoOnBottom()) {
				if (getDTable().tools().isVisible()
						&& getDTable().tools().isOnBottom()) {
					tableTools.after($("<div/>").id("dt-filter-info")
							.addClass("dt-filter-info")
							.html(filterHtml.toString()));
				} else {
					mainContainerWrapper.after($("<div/>").id("dt-filter-info")
							.addClass("dt-filter-info")
							.html(filterHtml.toString()));
				}
			}
		}
		filterHtml = new String();
	}

	public void unmarkFilter(int column) {
		String filterId = "td [id$=':" + column + "']";
		$(getDTable().isFrozen(column) ? frozenBodyId : bodyId).children("tr")
				.children(filterId).removeClass("filter");
	}

	public boolean isFilterEnabled(int column) {
		if (getDTable().getAllRows() > 1) {
			if (!getDTable().isFrozen(column)) {
				return (getDTable().config().isEnabledFilteredTable() && getDTable()
						.config().isEnabledContextMenuForFilteredTable());
			} else {
				return (getDTable().config().isEnabledFilteredFrozenTable() && getDTable()
						.config().isEnabledContextMenuForFilteredTable());
			}
		}
		return false;
	}

	public boolean isFiltered(int column) {
		return (columnsFilter.containsKey(column));
	}

	public boolean filterOperatorExists(int column, String operator) {
		if (isFiltered(column)) {
			List<Object> filters = columnsFilter.get(column);
			for (Object filter : filters) {
				if (((String[]) filter)[0].equals(operator)) {
					return true;
				}
			}
		}
		return false;
	}

	public boolean filterExists(int column, String filter) {
		if (isFiltered(column)) {
			List<Object> filters = columnsFilter.get(column);
			for (Object tempFilter : filters) {
				if (((String[]) tempFilter)[1].equals(filter)) {
					return true;
				}
			}
		}
		return false;
	}

	public void newFilter(int column, int row, String operator, String filter,
			String cellValue) {
		if (operator != null && !operator.isEmpty()) {
			getDTable()
					.sendMessage(
							"newFilter",
							(Integer.toString(column) + "_::_"
									+ Integer.toString(row) + "_::_" + operator
									+ "_::_" + filter + "_::_" + cellValue),
							true);
		}
	}

	public void removeFilter(int column) {
		if (isFiltered(column) || getDTable().search().isSearched(column)) {
			unmarkFilter(column);
			getDTable().search().emptySearch(column);
			getDTable().sendMessage("removeFilter", Integer.toString(column),
					true);
		}
	}

	public void markFilter() {
		if (((getDTable().config().isEnabledFilteredFrozenTable() || getDTable()
				.config().isEnabledFilteredTable()) && !columnsFilter.isEmpty())
				|| (getDTable().config().isEnabledSearch() && !getDTable()
						.search().isEmpty())) {
			List<Integer> columns = new ArrayList<Integer>(
					columnsFilter.keySet());
			columns.addAll(getDTable().search().getSearchColumns());
			Set<Integer> colSet = new HashSet<Integer>(columns);
			columns = new ArrayList<Integer>(colSet);
			Collections.sort(columns);

			for (Integer column : columns) {
				String filterId = "td [id$=':" + column + "']";
				$(getDTable().isFrozen(column) ? frozenBodyId : bodyId)
						.children("tr").children(filterId).addClass("filter");
			}
		}
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
			frozenBodyId += getDTable().getId();
			bodyId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		columnsFilter.clear();
		if (uidl.hasAttribute("newFilter")) {
			setFilter(uidl.getStringArrayAttribute("newFilter"));
		}

		if (uidl.hasAttribute("filterHtml")) {
			setFilterHtml(uidl.getStringAttribute("filterHtml"));
		}
	}

	@Override
	public void load() {
		setIds();
		enableEventsHandler();
	}

	@Override
	public void unload() {
		disableEventsHandler();
		$(mainId + " div#dt-filter-info").remove();
	}

	public void apply() {
		if (((getDTable().config().isEnabledFilteredFrozenTable() || getDTable()
				.config().isEnabledFilteredTable()) && !columnsFilter.isEmpty())
				|| (getDTable().config().isEnabledSearch() && !getDTable()
						.search().isEmpty())) {
			showFilterHtml();
		} else {
			unload();
		}
	}

	@Override
	public void enableEventsHandler() {
	}

	@Override
	public void disableEventsHandler() {
	}

}
