/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.List;

import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.vaadin.terminal.gwt.client.UIDL;

public class Order extends TableComponent {

	private String mainId = "div#v-dtable";

	private List<String> orders = new ArrayList<String>();

	public Order(VDTable dTable) {
		super(dTable);
	}

	private int getOrderColumn(String order) {
		return Integer.parseInt(order.split(" ")[0]);
	}

	private String getOrderType(String order) {
		return order.split(" ")[1];
	}

	private void setOrders(String[] orders) {
		if (orders != null && orders.length > 0) {
			this.orders.clear();
			if (!"none".equals(orders[0])) {
				for (String order : orders) {
					order = order.toLowerCase();
					int column = getOrderColumn(order);
					if (getDTable().isFrozen(column)
							&& getDTable().config()
									.isEnabledOrderedFrozenTable()) {
						this.orders.add(order);
					} else if (!getDTable().isFrozen(column)
							&& getDTable().config().isEnabledOrderedTable()) {
						this.orders.add(order);
					}
				}
			}
		}
	}

	public boolean isOrderEnabled(int column) {
		return ((getDTable().getAllRows() > 1) && (!getDTable()
				.isFrozen(column) ? getDTable().config().isEnabledOrderedTable()
				: getDTable().config().isEnabledOrderedFrozenTable()));
	}

	public int getSize(boolean isTableFrozen) {
		if (orders.isEmpty()) {
			return 0;
		} else {
			int columnsCounter = 0;
			for (int col = 0; col < orders.size(); col++) {
				int column = getOrderColumn(orders.get(col));
				if (isTableFrozen && getDTable().isFrozen(column)) {
					columnsCounter += 1;
				} else if (!isTableFrozen && !getDTable().isFrozen(column)) {
					columnsCounter += 1;
				}
			}
			return columnsCounter;
		}
	}

	public String getOrders() {
		StringBuilder builder = new StringBuilder();
		for (String order : this.orders) {
			builder.append(order.toUpperCase()).append("_:_");
		}
		return (builder.length() == 0 ? "none" : builder.substring(0,
				builder.length() - 3));
	}

	public void setOrder(GQuery td, String order) {
		if (td != null && !td.isEmpty()) {
			int columnId = Integer.parseInt(td.id().split(":")[2]);
			if (order == null) {
				for (int col = this.orders.size() - 1; col >= 0; col--) {
					int column = getOrderColumn(this.orders.get(col));
					if (getDTable().isFrozen(columnId)
							&& getDTable().isFrozen(column)) {
						this.orders.remove(col);
					} else if (!getDTable().isFrozen(columnId)
							&& !getDTable().isFrozen(column)) {
						this.orders.remove(col);
					}
				}
			} else {
				order = order.toLowerCase();
				int column = -1;
				for (int col = 0; col < this.orders.size(); col++) {
					if (getOrderColumn(this.orders.get(col)) == columnId) {
						column = col;
					}
				}

				if ("asc".equals(order) || "desc".equals(order)) {
					if (column >= 0) {
						this.orders.set(column, columnId + " " + order);
					} else {
						this.orders.add(columnId + " " + order);
					}
				} else if ("remove".equals(order) && column >= 0) {
					this.orders.remove(column);
				}
			}
		}
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		if (uidl.hasAttribute("order")) {
			setOrders(uidl.getStringArrayAttribute("order"));
		} else {
			unload();
		}
	}

	@Override
	public void load() {
		setIds();
		enableEventsHandler();
	}

	@Override
	public void unload() {
		disableEventsHandler();
	}

	public void apply() {
		if ((getDTable().config().isEnabledOrderedFrozenTable() || getDTable()
				.config().isEnabledOrderedTable())
				&& this.orders != null
				&& !this.orders.isEmpty()) {
			String tdId = "td [id='" + getDTable().getId() + "h:0:_']";
			for (String order : this.orders) {
				int column = getOrderColumn(order);
				if ((getDTable().config().isEnabledOrderedFrozenTable() && getDTable()
						.isFrozen(column))
						|| (getDTable().config().isEnabledOrderedTable() && !getDTable()
								.isFrozen(column))) {
					GQuery td = $(tdId.replace("_", "" + column));
					$(td).removeClass("asc", "desc")
							.children("div")
							.css(CSS.MARGIN_LEFT.with(Length.px(getDTable()
									.config().getHeaderCellMarginLeft())));
					$(td).addClass(getOrderType(order).toLowerCase())
							.children("div")
							.css(CSS.MARGIN_LEFT.with(Length.px(getDTable()
									.config().getHeaderCellMarginLeft()
									+ getDTable().config()
											.getHeaderCellMarginLeftOrdered())));
				}
			}
		} else {
			unload();
		}
	}

	@Override
	public void enableEventsHandler() {
	}

	@Override
	public void disableEventsHandler() {
	}
}
