/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.components.events.QuickSearchEvent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;

public class QuickSearch extends TableComponent {

	private String mainId = "div#v-dtable";
	private String quickSearchId = "#dt-quickSearch";

	private Boolean isQuickSearchMessageSend = false;
	private String focusValue = new String();
	private String eventValue = new String();

	private GQuery focusEventHandler = null;
	private GQuery blurEventHandler = null;
	private GQuery keyDownEventHandler = null;
	private GQuery keyUpEventHandler = null;

	public QuickSearch(VDTable dTable) {
		super(dTable);
	}

	private QuickSearch getThis() {
		return this;
	}

	private void activateFocusEventHandler(boolean enable) {
		if (enable) {
			activateFocusEventHandler(false);
			focusEventHandler = $(quickSearchId).focus(new Function() {
				public boolean f(Event event) {
					return QuickSearchEvent.focus(getThis(), event);
				}
			});
		} else if (!enable && focusEventHandler != null) {
			focusEventHandler.die();
			focusEventHandler = null;
		}
	}

	private void activateBlurEventHandler(boolean enable) {
		if (enable) {
			activateBlurEventHandler(false);
			blurEventHandler = $(quickSearchId).blur(new Function() {
				public boolean f(Event event) {
					return QuickSearchEvent.blur(getThis(), event);
				}
			});
		} else if (!enable && blurEventHandler != null) {
			blurEventHandler.die();
			blurEventHandler = null;
		}
	}

	private void activateKeyDownEventHandler(boolean enable) {
		if (enable) {
			activateKeyDownEventHandler(false);
			keyDownEventHandler = $(quickSearchId).keydown(new Function() {
				public boolean f(Event event) {
					return QuickSearchEvent.keyDown(getThis(), event);
				}
			});
		} else if (!enable && keyDownEventHandler != null) {
			keyDownEventHandler.die();
			keyDownEventHandler = null;
		}
	}

	private void activateKeyUpEventHandler(boolean enable) {
		if (enable) {
			activateKeyUpEventHandler(false);
			keyUpEventHandler = $(quickSearchId).keyup(new Function() {
				public boolean f(Event event) {
					return QuickSearchEvent.keyUp(getThis(), event);
				}
			});
		} else if (!enable && keyUpEventHandler != null) {
			keyUpEventHandler.die();
			keyUpEventHandler = null;
		}
	}

	private void show() {
		if (!isVisible()) {
			Element quickSearchDiv = DOM.createDiv();
			quickSearchDiv.setClassName("dt-quick-search");
			$(mainId).find("#dt-main-container").before(quickSearchDiv);

			Element quickSearchLabel = DOM.createLabel();
			quickSearchLabel.setInnerText(getDTable().translation().get(
					Translation.SEARCH)
					+ " ");
			quickSearchDiv.appendChild(quickSearchLabel);

			Element quickSearchBox = DOM.createInputText();
			quickSearchBox.setId(quickSearchId.substring(1));
			quickSearchDiv.appendChild(quickSearchBox);

			$(quickSearchId).parent().parent().css("-moz-border-radius", "")
					.css("-webkit-border-radius", "")
					.css("-khtml-border-radius", "").css("border-radius", "");
			enableEventsHandler();
		}
	}

	private boolean isVisible() {
		return ($(quickSearchId).size() > 0);
	}

	public void emptyQuickSearch() {
		focusValue = new String();
		eventValue = new String();
		$(quickSearchId).val("");
	}

	public Boolean isQuickSearchMessageSend() {
		return isQuickSearchMessageSend;
	}

	public void isQuickSearchMessageSend(Boolean isQuickSearchMessageSend) {
		this.isQuickSearchMessageSend = isQuickSearchMessageSend;
	}

	public String getFocusValue() {
		return focusValue;
	}

	public void setFocusValue(String focusValue) {
		this.focusValue = focusValue;
	}

	public String getEventValue() {
		return eventValue;
	}

	public void setEventValue(String eventValue) {
		this.eventValue = eventValue;
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
			quickSearchId += getDTable().getId();
		}
	}

	@Override
	public void load() {
		setIds();
		if (getDTable().config().isEnabledQuickSearch()
				&& getDTable().config().isEnabledQuickSearchVisibleAtStartup()) {
			show();
		} else {
			unload();
		}
	}

	@Override
	public void unload() {
		$(mainId).children("#dt-main-container-wrapper")
				.css("-moz-border-radius", "0")
				.css("-webkit-border-radius", "0")
				.css("-khtml-border-radius", "0").css("border-radius", "0");
		if (isVisible()) {
			$(quickSearchId).parent().remove();
			if ((!focusValue.isEmpty() && focusValue.length() > 0)
					|| (!eventValue.isEmpty() && eventValue.length() > 0)) {
				getDTable().sendMessage("quickSearch", "", true);
			}
		}
		disableEventsHandler();
	}

	public void toggle() {
		if (isVisible()) {
			unload();
		} else {
			show();
		}
	}

	@Override
	public void enableEventsHandler() {
		if (getDTable().config().isEnabledQuickSearch()) {
			activateFocusEventHandler(true);
			activateBlurEventHandler(true);
			activateKeyDownEventHandler(true);
			activateKeyUpEventHandler(true);
		}
	}

	@Override
	public void disableEventsHandler() {
		activateFocusEventHandler(false);
		activateBlurEventHandler(false);
		activateKeyDownEventHandler(false);
		activateKeyUpEventHandler(false);
	}

}
