/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;
import static com.google.gwt.query.client.plugins.Effects.Effects;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.dom.client.Style.Cursor;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;

public class TableTools extends TableComponent {

	private int durationStart = 200;
	private int durationStop = 500;
	private int delay = durationStop + 100;

	private String activeButton = null;
	private String absoluteHPosition = new String();
	private String absoluteVPosition = new String();
	private String marginPosition = new String();
	private String paddingPosition = new String();

	private String mainId = "div#v-dtable";
	private String tableToolsId = "div#dt-table-tools";

	private String actionId = "#dt-actionId";
	private String actionQuickSearchId = "#dt-actionQuickSearch";
	private String actionSearchId = "#dt-actionSearch";
	private String actionFreezeColumnsId = "#dt-actionFreezeColumns";
	private String actionHideColumnsId = "#dt-actionHideColumns";

	private String exportId = "#dt-exportId";
	private String exportCsvId = "#dt-exportCsv";
	private String exportMdbId = "#dt-exportMdb";
	private String exportPdfId = "#dt-exportPdf";
	private String exportXlsId = "#dt-exportXls";

	private Element tableToolsDiv = null;
	private GQuery mouseOverEventHandler = null;
	private GQuery mouseClickEventHandler = null;

	public TableTools(VDTable dTable) {
		super(dTable);
	}

	private void activateMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateMouseOverEventHandler(false);
			mouseOverEventHandler = $(tableToolsId).delegate("div",
					"mouseover", new Function() {
						public boolean f(Event event) {
							mouseover(event);
							return false;
						}
					}).delegate("div", "mouseout", new Function() {
				public boolean f(Event event) {
					mouseout(event);
					return false;
				}
			});
		} else if (!enable && mouseOverEventHandler != null) {
			mouseOverEventHandler.undelegate(tableToolsId, "mouseover")
					.undelegate(tableToolsId, "mouseout");
			mouseOverEventHandler = null;
		}
	}

	private void activateMouseClickEventHandler(boolean enable) {
		if (enable) {
			activateMouseClickEventHandler(false);
			mouseClickEventHandler = $(mainId).delegate(tableToolsId, "click",
					new Function() {
						public boolean f(Event event) {
							mouseclick(event);
							return false;
						}
					}).delegate(tableToolsId, "dblclick", new Function() {
				public boolean f(Event event) {
					return false;
				}
			});
		} else if (!enable && mouseClickEventHandler != null) {
			mouseClickEventHandler.undelegate(tableToolsId, "click")
					.undelegate(tableToolsId, "dblclick");
			mouseClickEventHandler = null;
		}
	}

	private int getCompleteWidth() {
		int width = 0;
		for (int element = 0; element < $(tableToolsDiv).children().size(); element++) {
			width += $(tableToolsDiv).children().eq(element).outerWidth(true);
		}
		return width;
	}

	private int getCompleteHeight(boolean includeMargin) {
		if (includeMargin) {
			$(tableToolsDiv).outerHeight(true);
		}
		return $(tableToolsDiv).height();
	}

	private int getMarginSize(GQuery e) {
		String margin = e.css(marginPosition);
		if (margin.endsWith("px")) {
			return Math.abs(Integer.parseInt(margin.substring(0,
					margin.indexOf("px"))));
		}
		return 0;
	}

	private int getPaddinSize(GQuery e) {
		String padding = e.css(paddingPosition);
		if (padding.endsWith("px")) {
			return Math.abs(Integer.parseInt(padding.substring(0,
					padding.indexOf("px"))));
		}
		return 0;
	}

	private int getAbsoluteVPositionSize(GQuery e) {
		String absolute = e.css(absoluteVPosition);
		if (absolute.endsWith("px")) {
			return Integer.parseInt(absolute.substring(0,
					absolute.indexOf("px")));
		}
		return 0;
	}

	private void appendButton(String id, String idTitle, String clazz) {
		String hAbsolute = getCompleteWidth() + "px";
		String vAbsolute = (-getCompleteHeight(true)) + "px";
		String animation = absoluteVPosition + ":+=" + getCompleteHeight(true)
				+ "px;";
		clazz += getDTable().config().getTableToolsPosition();
		$(tableToolsDiv).append(
				$("<div />").id(id).text(idTitle).addClass(clazz)
						.css(absoluteHPosition, hAbsolute)
						.css(absoluteVPosition, vAbsolute).as(Effects)
						.animate(animation, durationStop, new Function() {
							public void f(Element e) {
								int height = getCompleteHeight(false)
										- $(e).outerHeight(true);
								$(e).css(marginPosition, -height + "px")
										.css(paddingPosition,
												height + getPaddinSize($(e))
														+ "px")
										.css(absoluteVPosition, "");
							}
						}));
	}

	private void appendActionButtons() {
		if (getDTable().config().isEnabledHideUnhideColumns()) {
			appendButton(actionHideColumnsId.substring(1), getDTable()
					.translation().get(Translation.HIDE_UNHIDE_COLUMNS),
					"action");
		}

		if (getDTable().config().isEnabledFreezeUnfreezeColumns()) {
			appendButton(actionFreezeColumnsId.substring(1), getDTable()
					.translation().get(Translation.FREEZE_UNFREEZE_COLUMNS),
					"action");
		}

		if (getDTable().config().isEnabledSearchHide()) {
			appendButton(actionSearchId.substring(1), getDTable().translation()
					.get(Translation.SEARCH_IN_COLUMNS), "action");
		}

		if (getDTable().config().isEnabledQuickSearchHide()) {
			appendButton(actionQuickSearchId.substring(1), getDTable()
					.translation().get(Translation.QUICK_SEARCH), "action");
		}
	}

	private void appendExportButtons() {
		if (getDTable().exporter().isEnabledExportPdfFile()) {
			appendButton(exportPdfId.substring(1), getDTable().translation()
					.get(Translation.PDF_FILE), "action");
		}

		if (getDTable().exporter().isEnabledExportMdbFile()) {
			appendButton(exportMdbId.substring(1), getDTable().translation()
					.get(Translation.MS_ACCESS_FILE), "action");
		}

		if (getDTable().exporter().isEnabledExportXlsFile()) {
			appendButton(exportXlsId.substring(1), getDTable().translation()
					.get(Translation.MS_EXCEL_FILE), "action");
		}

		if (getDTable().exporter().isEnabledExportCsvFile()) {
			appendButton(exportCsvId.substring(1), getDTable().translation()
					.get(Translation.CSV_FILE), "action");
		}
	}

	private void mouseover(Event event) {
		String animation = absoluteVPosition + ":+=" + getMarginSize($(event))
				+ "px";
		if (($(event).is(actionId, exportId) && activeButton == null)
				|| !$(event).is(actionId, exportId)) {
			$(event).as(Effects).animate(animation, durationStart);
		}
	}

	private void mouseout(Event event) {
		String animation = absoluteVPosition + ":-=" + getMarginSize($(event))
				+ "px";
		if (($(event).is(actionId, exportId) && activeButton == null)
				|| !$(event).is(actionId, exportId)) {
			$(event).as(Effects).animate(animation, durationStop,
					new Function() {
						public void f(Element e) {
							$(e).css(absoluteVPosition, "");
						}
					});
		}
	}

	private void mouseclick(Event event) {
		// disable click events
		activateMouseClickEventHandler(false);

		final String id = DOM.eventGetTarget(event).getId();
		if (id.equals(actionId.substring(1))
				|| id.equals(exportId.substring(1))) {
			if (activeButton == null) {
				String opacity = "opacity:0.4;filter:alpha(opacity=40);";
				$(event).children(actionId, exportId).not("#" + id)
						.css(CSS.CURSOR.with(Cursor.DEFAULT)).as(Effects)
						.animate(opacity, durationStop);
				if (id.equals(actionId.substring(1))) {
					appendActionButtons();
				} else if (id.equals(exportId.substring(1))) {
					appendExportButtons();
				}
				activeButton = id;
			} else if (id.equals(activeButton)) {
				String opacity = "opacity:1.0;filter:alpha(opacity=100);";
				String animation = absoluteVPosition + ":-="
						+ getCompleteHeight(true) + "px;";
				$(event).children(actionId, exportId).not("#" + id).as(Effects)
						.animate(opacity, durationStop, new Function() {
							public void f(Element e) {
								$(e).css(CSS.CURSOR.getCssName(), "")
										.css("opacity", "").css("filter", "");
							}
						});
				$(event).children("div").not(actionId, exportId).as(Effects)
						.animate(animation, durationStop, new Function() {
							public void f(Element e) {
								$(e).remove();
							}
						});
				activeButton = null;
			}
		} else {
			// reset table tool's buttons
			String opacity = "opacity:1.0;filter:alpha(opacity=100);";
			$(event).children(actionId, exportId).as(Effects)
					.animate(opacity, durationStop).each(new Function() {
						public void f(Element e) {
							String animation = absoluteVPosition + ":-="
									+ getAbsoluteVPositionSize($(e)) + "px;";
							$(e).as(Effects).animate(animation, durationStop,
									new Function() {
										public void f(Element e) {
											$(e).css(absoluteVPosition, "")
													.css(CSS.CURSOR
															.getCssName(), "")
													.css("opacity", "")
													.css("filter", "");
										}
									});
						}
					});
			String animation = absoluteVPosition + ":-="
					+ getCompleteHeight(true) + "px;";
			$(event).children("div").not(actionId, exportId).as(Effects)
					.animate(animation, durationStop, new Function() {
						public void f(Element e) {
							$(e).remove();
						}
					});
			activeButton = null;
		}

		// enable click events
		$(event).delay(delay, new Function() {
			public void f() {
				activateMouseClickEventHandler(true);
				// table tool's actions
				if (id.equals(exportCsvId.substring(1))) {
					getDTable().sendMessage("export", "csv", false);
				} else if (id.equals(exportMdbId.substring(1))) {
					getDTable().sendMessage("export", "mdb", false);
				} else if (id.equals(exportPdfId.substring(1))) {
					getDTable().sendMessage("export", "pdf", false);
				} else if (id.equals(exportXlsId.substring(1))) {
					getDTable().sendMessage("export", "xls", false);
				} else if (id.equals(actionQuickSearchId.substring(1))) {
					getDTable().quickSearch().toggle();
					getDTable().forceUpdateClientLayout();
				} else if (id.equals(actionSearchId.substring(1))) {
					getDTable().search().toggle();
					getDTable().forceUpdateClientLayout();
				} else if (id.equals(actionFreezeColumnsId.substring(1))) {
					getDTable().sendMessage("columnsAction", "freeze", false);
				} else if (id.equals(actionHideColumnsId.substring(1))) {
					getDTable().sendMessage("columnsAction", "hide", false);
				}
			}
		});
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
			tableToolsId += getDTable().getId();

			actionId += getDTable().getId();
			actionQuickSearchId += getDTable().getId();
			actionSearchId += getDTable().getId();
			actionFreezeColumnsId += getDTable().getId();
			actionHideColumnsId += getDTable().getId();

			exportId += getDTable().getId();
			exportCsvId += getDTable().getId();
			exportMdbId += getDTable().getId();
			exportPdfId += getDTable().getId();
			exportXlsId += getDTable().getId();
		}
	}

	public boolean isVisible() {
		return !$(tableToolsDiv).isEmpty();
	}

	public boolean isOnTop() {
		return (!$(tableToolsDiv).isEmpty() && getDTable().config()
				.getTableToolsPosition().contains("top"));
	}

	public boolean isOnBottom() {
		return (!$(tableToolsDiv).isEmpty() && getDTable().config()
				.getTableToolsPosition().contains("bottom"));
	}

	@Override
	public void load() {
		setIds();

		if (DOM.getElementById(tableToolsId.substring(4)) == null) {
			tableToolsDiv = DOM.createDiv();
			tableToolsDiv.setId(tableToolsId.substring(4));
			tableToolsDiv.setClassName("dt-table-tools");

			if (getDTable().config().getTableToolsPosition().contains("left")) {
				absoluteHPosition = CSS.LEFT.getCssName();
			} else {
				absoluteHPosition = CSS.RIGHT.getCssName();
			}
			if (getDTable().config().getTableToolsPosition().contains("bottom")) {
				absoluteVPosition = CSS.TOP.getCssName();
				marginPosition = CSS.MARGIN_TOP.getCssName();
				paddingPosition = CSS.PADDING_TOP.getCssName();
				$(mainId).find("#dt-main-container-wrapper").after(
						$(tableToolsDiv));
			} else {
				absoluteVPosition = CSS.BOTTOM.getCssName();
				marginPosition = CSS.MARGIN_BOTTOM.getCssName();
				paddingPosition = CSS.PADDING_BOTTOM.getCssName();
				$(mainId).find("#dt-main-container-wrapper").before(
						$(tableToolsDiv));
			}

			if (getDTable().config().isEnabledHideUnhideColumns()
					|| getDTable().config().isEnabledFreezeUnfreezeColumns()
					|| getDTable().config().isEnabledSearchHide()
					|| getDTable().config().isEnabledQuickSearchHide()) {
				appendButton(actionId.substring(1), getDTable().translation()
						.get(Translation.ACTION), "tool");
			}

			if (getDTable().exporter().isEnabledExportCsvFile()
					|| getDTable().exporter().isEnabledExportMdbFile()
					|| getDTable().exporter().isEnabledExportPdfFile()
					|| getDTable().exporter().isEnabledExportXlsFile()) {
				appendButton(exportId.substring(1), getDTable().translation()
						.get(Translation.EXPORT), "tool");
			}

			if ($(tableToolsDiv).children(actionId, exportId).size() == 0) {
				unload();
			}
		}
	}

	@Override
	public void unload() {
		disableEventsHandler();
		$(tableToolsId).remove();
	}

	@Override
	public void enableEventsHandler() {
		activateMouseOverEventHandler(true);
		activateMouseClickEventHandler(true);
	}

	@Override
	public void disableEventsHandler() {
		activateMouseOverEventHandler(false);
		activateMouseClickEventHandler(false);
	}

}
