/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.events;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.components.Search;

import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.user.client.Event;

public class SearchEvent {

	private static boolean canSendSearch(Search search, int column,
			String searchString) {
		if ("".equals(searchString.trim()) && !searchString.isEmpty()) {
			return false;
		} else if (search.getDTable().config().isColumnFormatted(column)
				&& !"".equals(searchString.replaceAll("\\s+", " ").trim())) {
			String tempSearch = searchString.replaceAll("\\s+", " ").trim();
			while (tempSearch.startsWith("=") || tempSearch.startsWith("<")
					|| tempSearch.startsWith(">") || tempSearch.startsWith("-")
					|| tempSearch.startsWith("+") || tempSearch.startsWith(",")
					|| tempSearch.startsWith(".")) {
				tempSearch = tempSearch.substring(1);
			}
			return (tempSearch.trim().length() > 0);
		} else if (!search.getDTable().config().isColumnFormatted(column)
				&& !"".equals(searchString.replaceAll("\\s+", " ").trim())) {
			String tempSearch = searchString.replaceAll("\\s+", " ").trim();
			while (tempSearch.startsWith("%") || tempSearch.startsWith("\"")
					|| tempSearch.startsWith("'")) {
				tempSearch = tempSearch.substring(1);
			}
			return (tempSearch.trim().length() > 0);
		}
		return true;
	}

	private static void checkSearchField(Search search, int column, Event event) {
		if (!canSendSearch(search, column, $(event).val())) {
			$(event).addClass("error");
		} else if (search.isSearchMarked()
				&& !search.getMarkedValue().isEmpty()
				&& search.getMarkedValue().equals($(event).val())) {
			$(event).addClass("error");
		} else {
			$(event).removeClass("error");
		}
	}

	private static String prepareSearchMessage(Search search, int column,
			String searchString) {
		if (searchString != null && !searchString.trim().isEmpty()) {
			searchString = searchString.replaceAll("\\s+", " ").trim();
			if (search.getDTable().config().isColumnFormatted(column)) {
				if (searchString.startsWith("<=")
						|| searchString.startsWith(">=")
						|| searchString.startsWith("<>")) {
					searchString.replace("<=", "<= ").replace(">=", ">= ")
							.replace("<>", "<> ");
				} else if (searchString.startsWith("=")
						|| searchString.startsWith("<")
						|| searchString.startsWith(">")) {
					searchString.replace("<", "< ").replace(">", "> ")
							.replace("=", "= ");
				} else {
					searchString = "= " + searchString;
				}
			}
			searchString = searchString.replaceAll("\\s+", " ").trim();
		}
		return (column + "_::_" + searchString);
	}

	public static void mouseOver(Search search, String headerId, String id) {
		// expanded header cell
		String[] ids = id.split(":");
		TableEvent.cellMouseOver(search.getDTable(), $(headerId + " tr")
				.children("td [id='" + ids[0] + "h:0:" + ids[2] + "']"), null);
	}

	public static void mouseOut(Search search, String headerId, String id) {
		// expanded header cell
		String[] ids = id.split(":");
		TableEvent.headerCellMouseOut(headerId + " tr", ids[0], ids[2], null);
	}

	public static boolean keyDown(Search search, Event event) {
		if (!search.getDTable().sendingMessage()) {
			if (event.getKeyCode() == KeyCodes.KEY_TAB) {
				$(event).blur();
				return false;
			}
		}
		return true;
	}

	public static boolean keyUp(Search search, Event event) {
		if (!search.getDTable().sendingMessage()) {
			String[] id = $(event).id().split(":");
			if (id != null && id.length == 4 && id[3].equals("2")) {
				int column = Integer.parseInt(id[2]);
				String value = $(event).val();
				checkSearchField(search, column, event);

				if (event.getKeyCode() == KeyCodes.KEY_ESCAPE) {
					search.setFocusValue(search.getInitValue());
					$(event).blur();
				} else if (event.getKeyCode() == KeyCodes.KEY_ENTER) {
					if (!search.getDTable().config().isEnabledSearchImmediate()
							&& !search.isSearchMessageSend()
							&& canSendSearch(search, column, value)
							&& !search.getFocusValue().equals(value)
							&& (!search.getEventValue().equals(value) || ""
									.equals(search.getEventValue()))) {
						search.sendSearchMessage(true);
						search.setEventValue(value);
						search.getDTable().sendMessage("search",
								prepareSearchMessage(search, column, value),
								true);
					}

					if (!search.isSearchMarked()) {
						if (canSendSearch(search, column, value)) {
							if (search.getDTable().config()
									.isEnabledSearchImmediate()) {
								value = search.getSearchValue(column);
								search.setEventValue(value);
							}
							search.setFocusValue(value);
						}
						$(event).blur();
					}
				} else if (search.getDTable().config()
						.isEnabledSearchImmediate()
						&& event.getKeyCode() != KeyCodes.KEY_CTRL
						&& event.getKeyCode() != KeyCodes.KEY_ALT
						&& event.getKeyCode() != KeyCodes.KEY_LEFT
						&& event.getKeyCode() != KeyCodes.KEY_RIGHT
						&& event.getKeyCode() != KeyCodes.KEY_UP
						&& event.getKeyCode() != KeyCodes.KEY_DOWN
						&& canSendSearch(search, column, value)) {
					if (!search.getEventValue().equals(value)) {
						search.sendSearchMessage(true);
						search.setEventValue(value);
						search.getDTable().sendMessage("search",
								prepareSearchMessage(search, column, value),
								true);
					}
				}
				return false;
			}
		}
		return true;
	}

	public static boolean focus(Search search, Event event) {
		String[] id = $(event).id().split(":");
		if (id != null && id.length == 4 && id[3].equals("2")) {
			int column = Integer.parseInt(id[2]);
			if (search.getDTable().isFrozen(column)
					&& !search.getDTable().config()
							.isEnabledFilteredFrozenTable()) {
				$(event).blur();
				return true;
			}
			search.setInitValue(column);
			search.setFocusValue($(event).val());
			search.setEventValue($(event).val());
			search.sendSearchMessage(false);
			search.resetMarkedValue();

			// expanded header cell
			String tr = null;
			if (search.getDTable().config()
					.isEnabledExpandedFrozenHeaderCellForActiveColumn()
					&& search.getDTable().isFrozen(column)) {
				tr = search.getFrozenHeaderId() + " tr";
			} else if (search.getDTable().config()
					.isEnabledExpandedHeaderCellForActiveColumn()
					&& !search.getDTable().isFrozen(column)) {
				tr = search.getHeaderId() + " tr";
			}
			if (tr != null) {
				TableEvent.headerCellMouseOut(tr, id[0], id[2], null);
				TableEvent.cellMouseOver(
						search.getDTable(),
						$(tr).children(
								"td [id='" + id[0] + "h:0:" + id[2] + "']"),
						"focus");
			}
		}
		return true;
	}

	public static boolean blur(Search search, Event event) {
		if ($(event).id() != null && $(event).id().split(":").length == 4) {
			String[] id = $(event).id().split(":");
			if (id[3].equals("2")) {
				String value = search.getFocusValue();
				if (search.isSearchMessageSend()
						&& !value.equals(search.getEventValue())) {
					search.setEventValue(value);
					search.getDTable().sendMessage(
							"search",
							prepareSearchMessage(search,
									Integer.parseInt(id[2]), value), true);
				}
				$(event).val(search.setFieldValue(value));
			}
			$(event).removeClass("error");

			// expanded header cell
			String tr = null;
			int column = Integer.parseInt(id[2]);
			if (search.getDTable().config()
					.isEnabledExpandedFrozenHeaderCellForActiveColumn()
					&& search.getDTable().isFrozen(column)) {
				tr = search.getFrozenHeaderId() + " tr";
			} else if (search.getDTable().config()
					.isEnabledExpandedHeaderCellForActiveColumn()
					&& !search.getDTable().isFrozen(column)) {
				tr = search.getHeaderId() + " tr";
			}
			if (tr != null) {
				TableEvent.headerCellMouseOut(tr, id[0], id[2], "focus");
			}
		}
		return true;
	}

}
