/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.query;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.dussan.vaadin.dtable.properties.filter.Filter;
import org.dussan.vaadin.dtable.properties.order.Order;
import org.dussan.vaadin.dtable.properties.order.OrderType;
import org.dussan.vaadin.dtable.query.elements.Field;
import org.dussan.vaadin.dtable.query.elements.Having;
import org.dussan.vaadin.dtable.query.elements.HavingType;
import org.dussan.vaadin.dtable.query.elements.OrderBy;
import org.dussan.vaadin.dtable.query.elements.OrderByType;
import org.dussan.vaadin.dtable.query.elements.Where;
import org.dussan.vaadin.dtable.query.elements.WhereType;

public class TableQuery implements Serializable {

	private static final long serialVersionUID = -8049470491981015573L;
	private Query query = null;
	private Query lockedQuery = null;

	public TableQuery() {
		query = new Query();
	}

	public Query getQuery() {
		return (lockedQuery == null ? query : lockedQuery);
	}

	public void setQuery(Query query) {
		this.query = query;
	}

	public Query getUnlockedQuery() {
		return query;
	}

	public boolean isQueryLocked() {
		return (lockedQuery != null);
	}

	public TableQuery lockQuery() {
		if (lockedQuery == null) {
			lockedQuery = Query.clone(query);
		}
		return this;
	}

	public TableQuery unlockQuery() {
		lockedQuery = null;
		return this;
	}

	public Query setProperties(Order[] orders, Filter[] filters,
			Filter[] quickSearch, int[] removedColumns) {
		if (!getQuery().getFields().isEmpty()) {
			Map<Integer, Field> fields = getQuery().getFields();

			if (orders != null && orders.length > 0) {
				for (Order order : orders) {
					OrderBy orderBy = new OrderBy(fields.get(order.getColumn())
							.getName()).orderBy(OrderType.ASCEDING.equals(order
							.getType()) ? OrderByType.ASCEDING
							: OrderByType.DESCEDING);
					getQuery().addOrderBy(orderBy);
				}
			}

			if (filters != null && filters.length > 0) {
				getQuery().setWhereMainType(WhereType.AND);
				List<Object> objects = new ArrayList<Object>();
				for (Filter filter : filters) {
					if (filter != null) {
						for (int operator = 0; operator < filter.getOperators().length; operator++) {
							objects.add(fields.get(filter.getColumn())
									.getName()
									+ " "
									+ filter.getOperators()[operator]
									+ " '"
									+ filter.getOriginalValues()[operator]
									+ "'");
						}
					}
				}
				getQuery().addWhereCondition(Where.AND(objects.toArray()));
			}

			if (quickSearch != null && quickSearch.length > 0) {
				getQuery().setHavingMainType(HavingType.OR);
				List<Object> objects = new ArrayList<Object>();
				for (Filter search : quickSearch) {
					if (search != null) {
						objects.add(fields.get(search.getColumn()).getName()
								+ " " + search.getOperators()[0] + " '"
								+ search.getOriginalValues()[0] + "'");
					}
				}
				getQuery().addHavingCondition(Having.OR(objects.toArray()));
			}

			for (int column : removedColumns) {
				if (fields.keySet().contains(column)) {
					Field field = fields.get(column);
					fields.put(
							column,
							new Field("''", field.getAlias(), field
									.getFunction()));
					getQuery().removeGroupBy(field.getAlias());
					getQuery().removeOrderBy(field.getAlias());
				}
			}
			getQuery().addFields(fields);
		}
		return getQuery();
	}
}
