/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.query.base;

public class ObjectsComparison<T> {

	private enum ComparisonType {
		EQUAL("="), //
		NOT_EQUAL("<>"), //
		LESS_THAN("<"), //
		GREATER_THAN(">"), //
		LESS_OR_EQUAL_THAN("<="), //
		GREATER_OR_EQUAL_THAN(">="), //
		IN("IN"), //
		NOT_IN("NOT IN"), //
		IS_NULL("IS NULL"), //
		IS_NOT_NULL("IS NOT NULL"), //
		LIKE("LIKE '?'"), //
		LEFT_LIKE("LIKE '%?'"), //
		LIKE_RIGHT("LIKE '?%'"), //
		LEFT_LIKE_RIGHT("LIKE '%?%'"), //
		NOT_LIKE("NOT LIKE '?'"), //
		NOT_LEFT_LIKE("NOT LIKE '%?'"), //
		NOT_LIKE_RIGHT("NOT LIKE '?%'"), //
		NOT_LEFT_LIKE_RIGHT("NOT LIKE '%?%'");

		private String comparisonType;

		private ComparisonType(String comparisonType) {
			this.comparisonType = comparisonType;
		}

		public String getComparisonType() {
			return comparisonType;
		}

		@Override
		public String toString() {
			return ("objects comparison type: " + name());
		}
	}

	private Object value1;
	private Object value2;
	private ComparisonType comparisonType;
	private String objectsComparison;

	public ObjectsComparison(String objectsComparison) {
		this.objectsComparison = objectsComparison;
	}

	private ObjectsComparison(Object value1, Object value2,
			ComparisonType comparisonType) {
		this.value1 = value1;
		this.value2 = value2;
		this.comparisonType = comparisonType;
	}

	private String processComparison() {
		if (objectsComparison != null && !objectsComparison.isEmpty()) {
			return objectsComparison;
		}

		comparisonType = comparisonType == null ? ComparisonType.EQUAL
				: comparisonType;
		switch (comparisonType) {
		case EQUAL:
		case GREATER_OR_EQUAL_THAN:
		case GREATER_THAN:
		case LESS_OR_EQUAL_THAN:
		case LESS_THAN:
		case NOT_EQUAL:
			return (value1 + comparisonType.getComparisonType() + value2);

		case IN:
		case NOT_IN:
			return (value1 + " " + comparisonType.getComparisonType() + " ("
					+ value2 + ")");

		case IS_NULL:
		case IS_NOT_NULL:
			return (value1 + " " + comparisonType.getComparisonType());

		case LEFT_LIKE:
		case LEFT_LIKE_RIGHT:
		case LIKE:
		case LIKE_RIGHT:
		case NOT_LEFT_LIKE:
		case NOT_LEFT_LIKE_RIGHT:
		case NOT_LIKE:
		case NOT_LIKE_RIGHT:
			return (value1 + comparisonType.getComparisonType().replace("?",
					value2.toString()));

		default:
			return null;
		}
	}

	public boolean isEmpty() {
		return !(value1 != null && !value1.toString().isEmpty()
				&& value2 != null && !value2.toString().isEmpty());
	}

	@SuppressWarnings("unchecked")
	public static <T> T eq(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.EQUAL));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T ge(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.GREATER_OR_EQUAL_THAN));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T gt(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.GREATER_THAN));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T le(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.LESS_OR_EQUAL_THAN));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T lt(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.LESS_THAN));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T ne(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.NOT_EQUAL));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T in(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.IN));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T notIn(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.NOT_IN));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T isNull(Object value) {
		if (value != null) {
			return ((T) new ObjectsComparison<T>(value, null,
					ComparisonType.IS_NULL));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T isNotNull(Object value) {
		if (value != null) {
			return ((T) new ObjectsComparison<T>(value, null,
					ComparisonType.IS_NOT_NULL));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T like(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.LIKE));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T leftLike(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.LEFT_LIKE));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T likeRight(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.LIKE_RIGHT));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T leftLikeRight(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.LEFT_LIKE_RIGHT));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T notLike(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.NOT_LIKE));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T notLeftLike(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.NOT_LEFT_LIKE));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T notLikeRight(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.NOT_LIKE_RIGHT));
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public static <T> T notLeftLikeRight(Object value1, Object value2) {
		if (value1 != null && value2 != null) {
			return ((T) new ObjectsComparison<T>(value1, value2,
					ComparisonType.NOT_LEFT_LIKE_RIGHT));
		}
		return null;
	}

	public static String AND(Object... objectsComparison) {
		if (objectsComparison == null || objectsComparison.length == 0) {
			return null;
		}

		StringBuilder comparisonBuilder = new StringBuilder();
		for (Object comparison : objectsComparison) {
			comparisonBuilder.append(" AND ");
			if (comparison instanceof ObjectsComparison<?>) {
				comparisonBuilder.append(((ObjectsComparison<?>) comparison)
						.processComparison());
			} else {
				comparisonBuilder.append(comparison.toString());
			}
		}
		return ("(" + comparisonBuilder.substring(" AND ".length()) + ")");
	}

	public static String OR(Object... objectsComparison) {
		if (objectsComparison == null || objectsComparison.length == 0) {
			return null;
		}

		StringBuilder comparisonBuilder = new StringBuilder();
		for (Object comparison : objectsComparison) {
			comparisonBuilder.append(" OR ");
			if (comparison instanceof ObjectsComparison<?>) {
				comparisonBuilder.append(((ObjectsComparison<?>) comparison)
						.processComparison());
			} else {
				comparisonBuilder.append(comparison.toString());
			}
		}
		return ("(" + comparisonBuilder.substring(" OR ".length()) + ")");
	}

	@Override
	public String toString() {
		return ("objects comparison: " + processComparison());
	}

}
