/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.query.elements;

public class Field {

	private String name;
	private String alias;
	private Functions function;

	public Field() {
	}

	public Field(Field field) {
		this.name = field.name;
		this.alias = field.alias;
		this.function = field.function;
	}

	public Field(String name) {
		this.name = name;
		this.alias = name;
		this.function = Functions.NONE;
	}

	public Field(String name, String alias) {
		this.name = name;
		this.alias = alias;
	}

	public Field(String name, Functions function) {
		this.name = name;
		this.alias = name;
		this.function = function;
	}

	public Field(String name, String alias, Functions function) {
		this.name = name;
		this.alias = alias;
		this.function = function;
	}

	public Field asAlias(String alias) {
		this.alias = alias;
		return this;
	}

	public Field addFieldFunction(Functions function) {
		this.function = function;
		return this;
	}

	public String getName() {
		return name;
	}

	public String getAlias() {
		return alias;
	}

	public Functions getFunction() {
		return function;
	}

	public String getSqlField() {
		function = function == null ? Functions.NONE : function;

		switch (function) {
		case COUNTER_DISTINCT:
			return (function.getFunction().replace("?", name) + " AS " + alias);

		case AVERAGE:
		case COUNTER:
		case MAXIMUM:
		case MINIMUM:
		case POPULATION_STANDARD_DEVIATION:
		case STANDARD_DEVIATION:
		case SUM:
			return (function.getFunction() + "(" + name + ")" + " AS " + alias);

		default:
			return (name + " AS " + alias);
		}
	}

	@Override
	public String toString() {
		StringBuilder string = new StringBuilder();
		string.append("name: " + name + ", ");
		string.append("alias: " + alias + ", ");
		string.append("function: "
				+ (function == null ? function.getFunction() : ""));
		return string.toString();
	}

}
