/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.query.elements;

public class TablesJoin {

	private TablesJoinType joinType;
	private String fromTable;
	private String toTable;
	private String toTableAlias;
	private String joinFields;

	public TablesJoin(TablesJoinType joinType, String fromTable,
			String toTable, String toTableAlias, String joinFields) {
		this.joinType = joinType;
		this.fromTable = fromTable;
		this.toTable = toTable;
		this.toTableAlias = toTableAlias;
		this.joinFields = joinFields;
	}

	public TablesJoin(TablesJoinType joinType, String fromTable,
			String toTable, String toTableAlias, JoinField joinField) {
		this.joinType = joinType;
		this.fromTable = fromTable;
		this.toTable = toTable;
		this.toTableAlias = toTableAlias;
		this.joinFields = joinField.getJoinFields();
	}

	public TablesJoin(TablesJoinType joinType, String fromTable,
			String toTable, String toTableAlias) {
		this.joinType = joinType;
		this.fromTable = fromTable;
		this.toTable = toTable;
		this.toTableAlias = toTableAlias;
	}

	public TablesJoin joinFields(String joinFields) {
		this.joinFields = joinFields;
		return this;
	}

	public TablesJoin joinFields(Object... joinFields) {
		this.joinFields = null;
		if (joinFields != null && joinFields.length > 0) {
			StringBuilder joinBuilder = new StringBuilder();
			for (Object join : joinFields) {
				joinBuilder.append(" AND ");
				if (join instanceof JoinField) {
					joinBuilder.append(((JoinField) join).getJoinFields());
				} else {
					joinBuilder.append(join.toString());
				}
			}
			this.joinFields = ("(" + joinBuilder.substring(" AND ".length()) + ")");
		}
		return this;
	}

	public String getTablesJoin() {
		if (joinType != null && fromTable != null && !fromTable.isEmpty()
				&& toTable != null && !toTable.isEmpty() && joinFields != null
				&& !joinFields.isEmpty()) {
			StringBuilder joinBuilder = new StringBuilder();
			joinBuilder.append(joinType.getTablesJoinType() + " ");
			joinBuilder.append(toTable + " " + toTableAlias + " ON ");
			joinBuilder.append(joinFields);
			return joinBuilder.toString().trim();
		}
		return null;
	}

	@Override
	public String toString() {
		return "tables join: " + getTablesJoin();
	}

}
