/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.translations.TranslatedWord;
import org.dussan.vaadin.dtable.windows.exporter.CsvExporter;
import org.dussan.vaadin.dtable.windows.exporter.MdbExporter;
import org.dussan.vaadin.dtable.windows.freeze.FreezeUnfreezeColumns;
import org.dussan.vaadin.dtable.windows.hide.HideUnhideColumns;

import com.vaadin.ui.Window;
import com.vaadin.ui.Window.Notification;

public class WindowsHelper {

	private DTable dTable = null;
	private Window window = null;

	public WindowsHelper(DTable dTable) {
		this.dTable = dTable;
	}

	public Window getWindow() {
		return window;
	}

	public void process(String type) {
		if (window != null && window.getParent() != null) {
			dTable.getApplication()
					.getMainWindow()
					.showNotification(
							WindowsType.HIDE.getType().equals(type) ? dTable
									.getTranslation()
									.get(dTable.getTableConfig().getLocale(),
											TranslatedWord.WINDOW_FOR_HIDING_UNHIDING_COLUMNS_DATA_IS_ALREADY_OPEN)
									: dTable.getTranslation()
											.get(dTable.getTableConfig()
													.getLocale(),
													TranslatedWord.WINDOW_FOR_EXPORTING_DATA_IS_ALREADY_OPEN),
							Notification.TYPE_ERROR_MESSAGE);
			return;
		}

		window = null;
		if (WindowsType.FREEZE.getType().equals(type)) {
			window = new FreezeUnfreezeColumns(dTable);
		} else if (WindowsType.HIDE.getType().equals(type)) {
			window = new HideUnhideColumns(dTable);
		} else if (WindowsType.CSV.getType().equals(type)) {
			window = new CsvExporter(dTable);
		} else if (WindowsType.MS_ACCESS.getType().equals(type)) {
			window = new MdbExporter(dTable);
		}

		if (window != null) {
			dTable.getApplication().getMainWindow().addWindow(window);
		} else {
			dTable.getApplication()
					.getMainWindow()
					.showNotification(
							dTable.getTranslation()
									.get(dTable.getTableConfig().getLocale(),
											TranslatedWord.IN_PREPARING_DATA_AN_ERROR_OCCURRED),
							Notification.TYPE_ERROR_MESSAGE);
		}
	}

}
