/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.hide;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.translations.TranslatedWord;

import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.ListSelect;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

public class HideUnhideColumns extends Window {

	private static final long serialVersionUID = 3771051434575477411L;
	private DTable dTable = null;
	private boolean isAttached = false;
	private String frozenAddition = null;

	private StringBuilder jsDisabledUnhideColumns = null;
	private StringBuilder jsDisabledHideColumns = null;
	private ListSelect unhiddenColumns = null;
	private ListSelect hiddenColumns = null;
	private Button closeButton = null;
	private Button okButton = null;
	private Button hideButton = null;
	private Button unhideButton = null;

	public HideUnhideColumns(DTable dTable) {
		this.dTable = dTable;
	}

	private DTable getDTable() {
		return dTable;
	}

	private void closeWindow() {
		setVisible(false);
		getParent().removeWindow(this);
	}

	private void setVisibleAndHiddenColumns() {
		unhiddenColumns.removeAllItems();
		if (!getDTable().getTableProperties().isNoFrozenColumn()) {
			for (int column : getDTable().getTableProperties()
					.getFrozenColumns()) {
				unhiddenColumns.addItem(getDTable().getTableProperties()
						.getColumnHeader(column) + frozenAddition);
			}
		}

		for (int column = 0; column < getDTable().getTableProperties()
				.getAllColumns(); column++) {
			if (!getDTable().getTableProperties().isFrozenColumn(column)
					&& !getDTable().getTableProperties().isHiddenColumn(column)) {
				unhiddenColumns.addItem(getDTable().getTableProperties()
						.getColumnHeader(column));
			}
		}

		hiddenColumns.removeAllItems();
		if (!getDTable().getTableProperties().isNoHiddenColumn()) {
			for (String column : getDTable().getTableProperties()
					.getHiddenColumnsString()) {
				int col = Integer.parseInt(column.substring(1));
				if (getDTable().getTableProperties().isHiddenColumnFrozen(col)) {
					hiddenColumns.addItem(getDTable().getTableProperties()
							.getColumnHeader(col) + frozenAddition);
				}
			}

			for (String column : getDTable().getTableProperties()
					.getHiddenColumnsString()) {
				int col = Integer.parseInt(column.substring(1));
				if (!getDTable().getTableProperties().isHiddenColumnFrozen(col)) {
					hiddenColumns.addItem(getDTable().getTableProperties()
							.getColumnHeader(col));
				}
			}
		}

		jsDisabledUnhideColumns = new StringBuilder();
		for (int column = 0; column < getDTable().getTableProperties()
				.getAllColumns(); column++) {
			String header = getDTable().getTableProperties().getColumnHeader(
					column);
			if (!getDTable().getTableConfig().canColumnUnhide(column)) {
				jsDisabledUnhideColumns.append(",'" + header + frozenAddition
						+ "'");
				jsDisabledUnhideColumns.append(",'" + header + "'");
			}
		}
		if (jsDisabledUnhideColumns.length() > 0) {
			jsDisabledUnhideColumns = new StringBuilder("["
					+ jsDisabledUnhideColumns.substring(1) + "]");
		}

		jsDisabledHideColumns = new StringBuilder();
		for (int column = 0; column < getDTable().getTableProperties()
				.getAllColumns(); column++) {
			String header = getDTable().getTableProperties().getColumnHeader(
					column);
			if (!getDTable().getTableConfig().canColumnHide(column)) {
				jsDisabledHideColumns.append(",'" + header + frozenAddition
						+ "'");
				jsDisabledHideColumns.append(",'" + header + "'");
			}
		}
		if (jsDisabledHideColumns.length() > 0) {
			jsDisabledHideColumns = new StringBuilder("["
					+ jsDisabledHideColumns.substring(1) + "]");
		}
	}

	private int isAtLeastOneColumnUnhidden(Collection<?> ids) {
		int unhiddenColumns = 0;
		if (getDTable().getTableConfig().isEnabledFrozenTable()
				&& getDTable().getTableProperties().getFrozenColumns() != null
				&& getDTable().getTableProperties().getFrozenColumns().length > 0) {
			for (Object id : ids) {
				String idString = id.toString();
				if (idString.endsWith(frozenAddition)) {
					unhiddenColumns += 1;
				}
			}
			unhiddenColumns = unhiddenColumns == 0 ? -2 : 0;
		}

		if (unhiddenColumns == 0) {
			for (Object id : ids) {
				String idString = id.toString();
				if (!idString.endsWith(frozenAddition)) {
					unhiddenColumns += 1;
				}
			}
			unhiddenColumns = unhiddenColumns == 0 ? -1 : unhiddenColumns;
		}

		return unhiddenColumns;
	}

	public void refreshDisabledColumns(String jsDisabledColumns, String id) {
		if (!jsDisabledColumns.isEmpty()) {
			StringBuilder js = new StringBuilder();
			js.append("setTimeout(function(){var s=");
			js.append(jsDisabledColumns);
			js.append(";var i=document.getElementById('");
			js.append(id);
			js.append("').children[0].options.length;for(var iii=0;iii<s.length;iii++){for(var ii=0;ii<i;ii++){if(s[iii]==document.getElementById('");
			js.append(id);
			js.append("').children[0].options[ii].text){document.getElementById('");
			js.append(id);
			js.append("').children[0].options[ii].disabled=true;}}}},150);");
			getApplication().getMainWindow().executeJavaScript(js.toString());
		}
	}

	@Override
	public void attach() {
		if (!isAttached) {
			frozenAddition = " ("
					+ getDTable().getTranslation().get(
							getDTable().getTableConfig().getLocale(),
							TranslatedWord.FROZEN) + ")";

			VerticalLayout windowLayout = (VerticalLayout) getContent();
			windowLayout.addStyleName("v-dtable-dialog");
			windowLayout.setMargin(false);
			windowLayout.setSpacing(false);

			setCaption(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.HIDE_UNHIDE_COLUMNS));
			center();
			setClosable(false);
			setDraggable(false);
			setResizable(false);
			setModal(true);
			setWidth(600, UNITS_PIXELS);

			HorizontalLayout layout = new HorizontalLayout();
			layout.addStyleName("context");
			layout.setWidth(100, UNITS_PERCENTAGE);
			layout.setMargin(true);
			layout.setSpacing(true);
			windowLayout.addComponent(layout);

			unhiddenColumns = new ListSelect(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.UNHIDDEN_COLUMNS));
			unhiddenColumns.setDebugId("unhiddenColumns");
			unhiddenColumns.setWidth(100, UNITS_PERCENTAGE);
			unhiddenColumns.setNullSelectionAllowed(true);
			unhiddenColumns.setMultiSelect(true);
			unhiddenColumns.setImmediate(true);
			unhiddenColumns.setRows(25);
			layout.addComponent(unhiddenColumns);

			VerticalLayout actionLayout = new VerticalLayout();
			actionLayout.setCaption("");
			actionLayout.setSizeUndefined();
			actionLayout.setMargin(false);
			actionLayout.setSpacing(false);
			layout.addComponent(actionLayout);

			hiddenColumns = new ListSelect(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.HIDDEN_COLUMNS));
			hiddenColumns.setDebugId("hiddenColumns");
			hiddenColumns.setWidth(100, UNITS_PERCENTAGE);
			hiddenColumns.setNullSelectionAllowed(true);
			hiddenColumns.setMultiSelect(true);
			hiddenColumns.setImmediate(true);
			hiddenColumns.setRows(25);
			layout.addComponent(hiddenColumns);
			layout.setExpandRatio(unhiddenColumns, 1.0f);
			layout.setExpandRatio(hiddenColumns, 1.0f);
			setVisibleAndHiddenColumns();

			hideButton = new Button(">>", new ClickListener() {
				private static final long serialVersionUID = -1577513961709444468L;

				@Override
				public void buttonClick(ClickEvent event) {
					Object values = unhiddenColumns.getValue();
					if (values != null) {
						for (Object value : (Set<?>) values) {
							unhiddenColumns.removeItem(value);
							hiddenColumns.addItem(value);
						}
					}
					unhiddenColumns.select(null);
					refreshDisabledColumns(jsDisabledUnhideColumns.toString(),
							hiddenColumns.getDebugId());
					refreshDisabledColumns(jsDisabledHideColumns.toString(),
							unhiddenColumns.getDebugId());
				}
			});
			actionLayout.addComponent(hideButton);

			unhideButton = new Button("<<", new ClickListener() {
				private static final long serialVersionUID = 1108475605502334432L;

				@Override
				public void buttonClick(ClickEvent event) {
					Object values = hiddenColumns.getValue();
					if (values != null) {
						for (Object value : (Set<?>) values) {
							hiddenColumns.removeItem(value);
							unhiddenColumns.addItem(value);
						}
					}
					hiddenColumns.select(null);
					refreshDisabledColumns(jsDisabledUnhideColumns.toString(),
							hiddenColumns.getDebugId());
					refreshDisabledColumns(jsDisabledHideColumns.toString(),
							unhiddenColumns.getDebugId());
				}
			});
			actionLayout.addComponent(unhideButton);

			HorizontalLayout space1 = new HorizontalLayout();
			space1.setWidth(100, UNITS_PERCENTAGE);
			space1.setHeight(6, UNITS_PIXELS);
			windowLayout.addComponent(space1);
			windowLayout.setComponentAlignment(space1, Alignment.BOTTOM_CENTER);

			HorizontalLayout buttonLayout = new HorizontalLayout();
			buttonLayout.setWidth(100, UNITS_PERCENTAGE);
			buttonLayout.setMargin(false);
			buttonLayout.setSpacing(true);
			windowLayout.addComponent(buttonLayout);
			windowLayout.setComponentAlignment(buttonLayout,
					Alignment.BOTTOM_CENTER);

			HorizontalLayout space2 = new HorizontalLayout();
			space2.setWidth(100, UNITS_PERCENTAGE);
			space2.setHeight(6, UNITS_PIXELS);
			windowLayout.addComponent(space2);
			windowLayout.setComponentAlignment(space2, Alignment.BOTTOM_CENTER);

			okButton = new Button(getDTable().getTranslation()
					.get(getDTable().getTableConfig().getLocale(),
							TranslatedWord.OK), new ClickListener() {
				private static final long serialVersionUID = -1577513961709444468L;

				@Override
				public void buttonClick(ClickEvent event) {
					List<Integer> columns = new ArrayList<Integer>();
					if (hiddenColumns.getItemIds() != null) {
						for (Object id : (Collection<?>) hiddenColumns
								.getItemIds()) {
							String idString = id.toString();
							if (idString.endsWith(frozenAddition)) {
								idString = idString.substring(0,
										idString.lastIndexOf(frozenAddition));
							}
							columns.add(getDTable().getTableProperties()
									.getColumnHeaderIndex(idString));
						}
					}

					switch (isAtLeastOneColumnUnhidden(unhiddenColumns
							.getItemIds())) {
					case -2:
						getApplication()
								.getMainWindow()
								.showNotification(
										getDTable()
												.getTranslation()
												.get(getDTable()
														.getTableConfig()
														.getLocale(),
														TranslatedWord.AT_LEAST_ONE_FROZEN_COLUMN_HAVE_TO_BE_UNHIDDEN),
										Notification.TYPE_ERROR_MESSAGE);
						break;
					case -1:
						getApplication()
								.getMainWindow()
								.showNotification(
										getDTable()
												.getTranslation()
												.get(getDTable()
														.getTableConfig()
														.getLocale(),
														TranslatedWord.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNHIDDEN),
										Notification.TYPE_ERROR_MESSAGE);
						break;
					default:
						getDTable().getTableProperties().resetHiddenColumns();
						if (!columns.isEmpty()) {
							getDTable().getTableProperties().setHiddenColumns(
									ArrayUtils.toPrimitive(columns
											.toArray(new Integer[] {})));
						}
						closeWindow();
						getDTable().refreshVisibleData("hideUnhideColumns",
								true);
						break;
					}
				}
			});
			buttonLayout.addComponent(okButton);
			buttonLayout
					.setComponentAlignment(okButton, Alignment.MIDDLE_RIGHT);

			closeButton = new Button(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.CLOSE), new ClickListener() {
				private static final long serialVersionUID = -2066785165595149511L;

				@Override
				public void buttonClick(ClickEvent event) {
					closeWindow();
				}
			});
			buttonLayout.addComponent(closeButton);
			buttonLayout.setComponentAlignment(closeButton,
					Alignment.MIDDLE_LEFT);

			refreshDisabledColumns(jsDisabledUnhideColumns.toString(),
					hiddenColumns.getDebugId());
			refreshDisabledColumns(jsDisabledHideColumns.toString(),
					unhiddenColumns.getDebugId());
			isAttached = true;
		}
		super.attach();
	}

	@Override
	public void detach() {
		if (isAttached) {
			removeAllComponents();
			isAttached = false;
		}
		super.detach();
	}

}
